import logging
import random
from typing import Any, Callable, cast

import humanize
from .backend import FunctionSpec, compile_prompt_to_md, query
from .interpreter import ExecutionResult
from .journal import Journal, Node
from .utils import data_preview
from .utils.config import Config
from .utils.metric import MetricValue, WorstMetricValue
from .utils.response import extract_code, extract_text_up_to_code, wrap_code

logger = logging.getLogger("aide")


ExecCallbackType = Callable[[str, bool], ExecutionResult]

review_func_spec = FunctionSpec(
    name="submit_review",
    json_schema={
        "type": "object",
        "properties": {
            "is_bug": {
                "type": "boolean",
                "description": "true if the output log shows that the execution failed or has some bug, otherwise false.",
            },
            "summary": {
                "type": "string",
                "description": "if there is a bug, propose a fix. Otherwise, write a short summary (2-3 sentences) describing the empirical findings.",
            },
            "metric": {
                "type": "number",
                "description": "If the code ran successfully, report the value of the validation metric. Otherwise, leave it null.",
            },
            "lower_is_better": {
                "type": "boolean",
                "description": "true if the metric should be minimized (i.e. a lower metric value is better, such as with MSE), false if the metric should be maximized (i.e. a higher metric value is better, such as with accuracy).",
            },
        },
        "required": ["is_bug", "summary", "metric", "lower_is_better"],
    },
    description="Submit a review evaluating the output of the training script.",
)


class Agent:
    def __init__(
        self,
        task_desc: str,
        cfg: Config,
        journal: Journal,
    ):
        super().__init__()
        self.task_desc = task_desc
        self.cfg = cfg
        self.acfg = cfg.agent
        self.journal = journal
        self.data_preview: str | None = None

    def search_policy(self) -> Node | None:
        """Select a node to work on (or None to draft a new node)."""
        search_cfg = self.acfg.search

        # initial drafting
        if len(self.journal.draft_nodes) < search_cfg.num_drafts:
            logger.debug("[search policy] drafting new node (not enough drafts)")
            return None

        # debugging
        if random.random() < search_cfg.debug_prob:
            # nodes that are buggy + leaf nodes + debug depth < max debug depth
            debuggable_nodes = [
                n
                for n in self.journal.buggy_nodes
                if (n.is_leaf and n.debug_depth <= search_cfg.max_debug_depth)
            ]
            if debuggable_nodes:
                logger.debug("[search policy] debugging")
                return random.choice(debuggable_nodes)
            logger.debug("[search policy] not debugging by chance")

        # back to drafting if no nodes to improve
        good_nodes = self.journal.good_nodes
        if not good_nodes:
            logger.debug("[search policy] drafting new node (no good nodes)")
            return None

        # greedy
        greedy_node = self.journal.get_best_node()
        logger.debug("[search policy] greedy node selected")
        return greedy_node

    @property
    def _prompt_environment(self):
        pkgs = [
            "numpy",
            "pandas",
            "scikit-learn",
            "statsmodels",
            "xgboost",
            "lightGBM",
            "torch",
            "torchvision",
            "torch-geometric",
            "bayesian-optimization",
            "timm",
        ]
        random.shuffle(pkgs)
        pkg_str = ", ".join([f"`{p}`" for p in pkgs])

        env_prompt = {
            "Installed Packages": f"Your solution can use any relevant machine learning packages such as: {pkg_str}. Feel free to use any other packages too (all packages are already installed!). For neural networks we suggest using PyTorch rather than TensorFlow."
        }
        return env_prompt

    @property
    def _prompt_impl_guideline(self):
        impl_guideline = [
            "The code should **implement the proposed solution** and **print the value of the evaluation metric computed on a hold-out validation set**.",
            "The code should be a single-file python program that is self-contained and can be executed as-is.",
            "No parts of the code should be skipped, don't terminate the before finishing the script.",
            "Your response should only contain a single code block.",
            f"Be aware of the running time of the code, it should complete within {humanize.naturaldelta(self.cfg.exec.timeout)}.",
            'All the provided input data is stored in "./input" directory.',
            '**If there is test data provided for this task, please save the test predictions in a `submission.csv` file in the "./working" directory as described in the task description** This is extremely important since this file is used for grading/evaluation. DO NOT FORGET THE submission.csv file!',
            'You can also use the "./working" directory to store any temporary files that your code needs to create.',
        ]
        if self.acfg.expose_prediction:
            impl_guideline.append(
                "The implementation should include a predict() function, "
                "allowing users to seamlessly reuse the code to make predictions on new data. "
                "The prediction function should be well-documented, especially the function signature."
            )

        if self.acfg.k_fold_validation > 1:
            impl_guideline.append(
                f"The evaluation should be based on {self.acfg.k_fold_validation}-fold cross-validation but only if that's an appropriate evaluation for the task at hand."
            )

        return {"Implementation guideline": impl_guideline}

    @property
    def _prompt_resp_fmt(self):
        return {
            "Response format": (
                "Your response should be a brief outline/sketch of your proposed solution in natural language (3-5 sentences), "
                "followed by a single markdown code block (wrapped in ```) which implements this solution and prints out the evaluation metric. "
                "There should be no additional headings or text in your response. Just natural language text followed by a newline and then the markdown code block. "
            )
        }

    def plan_and_code_query(self, prompt, retries=3) -> tuple[str, str]:
        """Generate a natural language plan + code in the same LLM call and split them apart."""
        completion_text = None
        for _ in range(retries):
            completion_text = query(
                system_message=prompt,
                user_message=None,
                model=self.acfg.code.model,
                temperature=self.acfg.code.temp,
            )

            code = extract_code(completion_text)
            nl_text = extract_text_up_to_code(completion_text)

            if code and nl_text:
                # merge all code blocks into a single string
                return nl_text, code

            print("Plan + code extraction failed, retrying...")
        print("Final plan + code extraction attempt failed, giving up...")
        return "", completion_text  # type: ignore

    def _draft(self) -> Node:
        prompt: Any = {
            "Introduction": (
                "You are a Kaggle grandmaster attending a competition. "
                "In order to win this competition, you need to come up with an excellent and creative plan "
                "for a solution and then implement this solution in Python. We will now provide a description of the task."
            ),
            "Task description": self.task_desc,
            "Memory": self.journal.generate_summary(),
            "Instructions": {},
        }
        prompt["Instructions"] |= self._prompt_resp_fmt
        prompt["Instructions"] |= {
            "Solution sketch guideline": [
                "This first solution design should be relatively simple, without ensembling or hyper-parameter optimization.",
                "Take the Memory section into consideration when proposing the design,"
                " don't propose the same modelling solution but keep the evaluation the same.",
                "The solution sketch should be 3-5 sentences.",
                "Propose an evaluation metric that is reasonable for this task.",
                "Don't suggest to do EDA.",
                "The data is already prepared and available in the `./input` directory. There is no need to unzip any files.",
            ],
        }
        prompt["Instructions"] |= self._prompt_impl_guideline
        prompt["Instructions"] |= self._prompt_environment

        if self.acfg.data_preview:
            prompt["Data Overview"] = self.data_preview

        plan, code = self.plan_and_code_query(prompt)
        return Node(plan=plan, code=code)

    def _improve(self, parent_node: Node) -> Node:
        prompt: Any = {
            "Introduction": (
                "You are a Kaggle grandmaster attending a competition. You are provided with a previously developed "
                "solution below and should improve it in order to further increase the (test time) performance. "
                "For this you should first outline a brief plan in natural language for how the solution can be improved and "
                "then implement this improvement in Python based on the provided previous solution. "
            ),
            "Task description": self.task_desc,
            "Memory": self.journal.generate_summary(),
            "Instructions": {},
        }
        prompt["Previous solution"] = {
            "Code": wrap_code(parent_node.code),
        }

        prompt["Instructions"] |= self._prompt_resp_fmt
        prompt["Instructions"] |= {
            "Solution improvement sketch guideline": [
                "The solution sketch should be a brief natural language description of how the previous solution can be improved.",
                "You should be very specific and should only propose a single actionable improvement.",
                "This improvement should be atomic so that we can experimentally evaluate the effect of the proposed change.",
                "Take the Memory section into consideration when proposing the improvement.",
                "The solution sketch should be 3-5 sentences.",
                "Don't suggest to do EDA.",
            ],
        }
        prompt["Instructions"] |= self._prompt_impl_guideline

        plan, code = self.plan_and_code_query(prompt)
        return Node(
            plan=plan,
            code=code,
            parent=parent_node,
        )

    def _debug(self, parent_node: Node) -> Node:
        prompt: Any = {
            "Introduction": (
                "You are a Kaggle grandmaster attending a competition. "
                "Your previous solution had a bug, so based on the information below, you should revise it in order to fix this bug. "
                "Your response should be an implementation outline in natural language,"
                " followed by a single markdown code block which implements the bugfix/solution."
            ),
            "Task description": self.task_desc,
            "Previous (buggy) implementation": wrap_code(parent_node.code),
            "Execution output": wrap_code(parent_node.term_out, lang=""),
            "Instructions": {},
        }
        prompt["Instructions"] |= self._prompt_resp_fmt
        prompt["Instructions"] |= {
            "Bugfix improvement sketch guideline": [
                "You should write a brief natural language description (3-5 sentences) of how the issue in the previous implementation can be fixed.",
                "Don't suggest to do EDA.",
            ],
        }
        prompt["Instructions"] |= self._prompt_impl_guideline

        if self.acfg.data_preview:
            prompt["Data Overview"] = self.data_preview

        plan, code = self.plan_and_code_query(prompt)
        return Node(plan=plan, code=code, parent=parent_node)

    def update_data_preview(
        self,
    ):
        self.data_preview = data_preview.generate(self.cfg.workspace_dir)

    def step(self, exec_callback: ExecCallbackType):
        if not self.journal.nodes or self.data_preview is None:
            self.update_data_preview()

        parent_node = self.search_policy()
        logger.debug(f"Agent is generating code, parent node type: {type(parent_node)}")

        if parent_node is None:
            result_node = self._draft()
        elif parent_node.is_buggy:
            result_node = self._debug(parent_node)
        else:
            result_node = self._improve(parent_node)

        self.parse_exec_result(
            node=result_node,
            exec_result=exec_callback(result_node.code, True),
        )
        self.journal.append(result_node)

    def parse_exec_result(self, node: Node, exec_result: ExecutionResult):
        logger.info(f"Agent is parsing execution results for node {node.id}")

        node.absorb_exec_result(exec_result)

        prompt = {
            "Introduction": (
                "You are a Kaggle grandmaster attending a competition. "
                "You have written code to solve this task and now need to evaluate the output of the code execution. "
                "You should determine if there were any bugs as well as report the empirical findings."
            ),
            "Task description": self.task_desc,
            "Implementation": wrap_code(node.code),
            "Execution output": wrap_code(node.term_out, lang=""),
        }

        response = cast(
            dict,
            query(
                system_message=prompt,
                user_message=None,
                func_spec=review_func_spec,
                model=self.acfg.feedback.model,
                temperature=self.acfg.feedback.temp,
            ),
        )

        # if the metric isn't a float then fill the metric with the worst metric
        if not isinstance(response["metric"], float):
            response["metric"] = None

        node.analysis = response["summary"]
        node.is_buggy = (
            response["is_bug"]
            or node.exc_type is not None
            or response["metric"] is None
        )

        if node.is_buggy:
            node.metric = WorstMetricValue()
        else:
            node.metric = MetricValue(
                response["metric"], maximize=not response["lower_is_better"]
            )



import pandas as pd
from sklearn.model_selection import train_test_split
from sklearn.metrics import accuracy_score

# Load the data
train_df = pd.read_csv("./input/train.csv")
test_df = pd.read_csv("./input/test.csv")
sample_submission = pd.read_csv("./input/sample_submission.csv")

# Split the training data into training and validation sets
train_texts, val_texts, train_indices, val_indices = train_test_split(
    train_df["text"], train_df["index"], test_size=0.1, random_state=42
)

# Placeholder for the predictions
val_predictions = [0] * len(val_texts)

# TODO: Implement the decryption algorithm here
# For now, we are just using a placeholder prediction
# In a real scenario, this is where the decryption logic would be applied

# Evaluate the accuracy of the predictions
accuracy = accuracy_score(val_indices, val_predictions)
print(f"Validation accuracy: {accuracy}")

# Prepare the submission file
test_predictions = [0] * len(test_df)
submission = pd.DataFrame(
    {"ciphertext_id": test_df["ciphertext_id"], "index": test_predictions}
)
submission.to_csv("./working/submission.csv", index=False)



import pandas as pd
import lightgbm as lgb
from sklearn.model_selection import train_test_split
from sklearn.preprocessing import LabelEncoder
from sklearn.metrics import log_loss

# Load the data
train_data = pd.read_csv("./input/train.csv")
test_data = pd.read_csv("./input/test.csv")
sample_submission = pd.read_csv("./input/sample_submission.csv")

# Prepare the data
X = train_data.drop(["id", "target"], axis=1)
y = train_data["target"]
label_encoder = LabelEncoder()
y_encoded = label_encoder.fit_transform(y)
X_train, X_val, y_train, y_val = train_test_split(
    X, y_encoded, test_size=0.2, random_state=42
)

# Prepare the test set
X_test = test_data.drop(["id"], axis=1)

# Train the model
train_set = lgb.Dataset(X_train, label=y_train)
val_set = lgb.Dataset(X_val, label=y_val, reference=train_set)

params = {
    "objective": "multiclass",
    "num_class": 9,
    "metric": "multi_logloss",
    "boosting_type": "gbdt",
    "learning_rate": 0.1,
    "num_leaves": 31,
    "max_depth": -1,
    "verbose": -1,
}

model = lgb.train(
    params,
    train_set,
    valid_sets=[train_set, val_set],
    early_stopping_rounds=100,
    num_boost_round=1000,
    verbose_eval=100,
)

# Evaluate the model
val_predictions = model.predict(X_val)
val_log_loss = log_loss(y_val, val_predictions)
print(f"Validation Log Loss: {val_log_loss}")

# Make predictions on the test set
test_predictions = model.predict(X_test)

# Prepare submission file
submission = pd.DataFrame(
    test_predictions, columns=["Class_" + str(i + 1) for i in range(9)]
)
submission["id"] = test_data["id"]
submission = submission[["id"] + ["Class_" + str(i + 1) for i in range(9)]]
submission.to_csv("./working/submission.csv", index=False)



import pandas as pd
from catboost import CatBoostClassifier
from sklearn.model_selection import train_test_split
from sklearn.metrics import roc_auc_score

# Load the data
train_data = pd.read_csv("./input/train.csv")
test_data = pd.read_csv("./input/test.csv")

# Separate features and target
X = train_data.drop(["id", "target"], axis=1)
y = train_data["target"]
X_test = test_data.drop(["id"], axis=1)

# Identify categorical features
cat_features = [col for col in X.columns if X[col].dtype == "object"]

# Split the data into training and validation sets
X_train, X_val, y_train, y_val = train_test_split(X, y, test_size=0.2, random_state=42)

# Initialize the CatBoostClassifier with a smaller number of iterations for faster grid search
model = CatBoostClassifier(
    iterations=100,  # Reduced number of iterations for grid search
    learning_rate=0.1,
    depth=4,
    loss_function="Logloss",
    early_stopping_rounds=10,
    verbose=False,
)

# Fit the model on the training data
model.fit(X_train, y_train, cat_features=cat_features)

# Predict on the validation set
val_pred = model.predict_proba(X_val)[:, 1]

# Calculate the ROC AUC score
val_auc = roc_auc_score(y_val, val_pred)
print(f"Validation AUC: {val_auc}")

# Train the final model on the full dataset with more iterations
final_model = CatBoostClassifier(
    iterations=1000,  # Increased number of iterations for final training
    learning_rate=0.1,
    depth=4,
    loss_function="Logloss",
    early_stopping_rounds=10,
    verbose=False,
)

final_model.fit(X, y, cat_features=cat_features)

# Predict on the test set
test_pred = final_model.predict_proba(X_test)[:, 1]

# Save the predictions to a CSV file
submission = pd.DataFrame({"id": test_data["id"], "target": test_pred})
submission.to_csv("./working/submission.csv", index=False)



import pandas as pd
from sklearn.model_selection import train_test_split
from sklearn.metrics import roc_auc_score
from lightgbm import LGBMClassifier

# Load the data
train_data = pd.read_csv("./input/train.csv")
test_data = pd.read_csv("./input/test.csv")

# Separate features and target
X = train_data.drop(columns=["id", "claim"])
y = train_data["claim"]

# Handle missing values by imputing with median
X.fillna(X.median(), inplace=True)
test_data.fillna(test_data.median(), inplace=True)

# Split the data into training and validation sets
X_train, X_val, y_train, y_val = train_test_split(X, y, test_size=0.2, random_state=42)

# Initialize the LightGBM model
model = LGBMClassifier(random_state=42)

# Train the model
model.fit(X_train, y_train)

# Predict on the validation set
val_predictions = model.predict_proba(X_val)[:, 1]

# Calculate the ROC AUC score
val_auc = roc_auc_score(y_val, val_predictions)
print(f"Validation ROC AUC Score: {val_auc}")

# Predict on the test set
test_predictions = model.predict_proba(test_data.drop(columns=["id"]))[:, 1]

# Create the submission file
submission = pd.DataFrame({"id": test_data["id"], "claim": test_predictions})
submission.to_csv("./working/submission.csv", index=False)



import pandas as pd
from sklearn.ensemble import (
    GradientBoostingRegressor,
    RandomForestRegressor,
    StackingRegressor,
)
from sklearn.linear_model import LinearRegression, RidgeCV
from sklearn.model_selection import train_test_split
from sklearn.metrics import mean_absolute_error
from sklearn.preprocessing import StandardScaler

# Load the data
train_data = pd.read_csv("./input/train.csv")
test_data = pd.read_csv("./input/test.csv")

# Prepare the data
X = train_data.drop(["id", "yield"], axis=1)
y = train_data["yield"]
X_test = test_data.drop("id", axis=1)

# Split the data into training and validation sets
X_train, X_val, y_train, y_val = train_test_split(X, y, test_size=0.2, random_state=42)

# Scale the features
scaler = StandardScaler()
X_train_scaled = scaler.fit_transform(X_train)
X_val_scaled = scaler.transform(X_val)
X_test_scaled = scaler.transform(X_test)

# Initialize base models
estimators = [
    (
        "gbr",
        GradientBoostingRegressor(
            n_estimators=200, learning_rate=0.1, max_depth=4, random_state=42
        ),
    ),
    ("rf", RandomForestRegressor(n_estimators=200, random_state=42)),
    ("lr", LinearRegression()),
]

# Initialize the StackingRegressor with a RidgeCV final estimator
stacking_regressor = StackingRegressor(estimators=estimators, final_estimator=RidgeCV())

# Train the StackingRegressor on the scaled training data
stacking_regressor.fit(X_train_scaled, y_train)

# Predict on the scaled validation set using the StackingRegressor
y_val_pred = stacking_regressor.predict(X_val_scaled)

# Evaluate the model
mae = mean_absolute_error(y_val, y_val_pred)
print(f"Mean Absolute Error on validation set with StackingRegressor: {mae}")

# Train the StackingRegressor on the full scaled training data and predict on the scaled test set
stacking_regressor.fit(scaler.transform(X), y)
test_predictions = stacking_regressor.predict(X_test_scaled)

# Save the predictions to a CSV file
submission = pd.DataFrame({"id": test_data["id"], "yield": test_predictions})
submission.to_csv("./working/submission.csv", index=False)



import pandas as pd
import numpy as np
from sklearn.model_selection import KFold
from sklearn.metrics import roc_auc_score
from sklearn.feature_selection import RFECV
import lightgbm as lgb
from bayes_opt import BayesianOptimization

# Load the data
train_data = pd.read_csv("./input/train.csv")
test_data = pd.read_csv("./input/test.csv")

# Prepare the data
X = train_data.drop(["id", "defects"], axis=1)
y = train_data["defects"]
X_test = test_data.drop("id", axis=1)
test_ids = test_data["id"]

# Initialize LightGBM model with the best parameters from previous optimization
best_params = {
    "num_leaves": 31,
    "learning_rate": 0.05,
    "subsample": 0.8,
    "colsample_bytree": 0.8,
    "max_depth": 15,
    "reg_alpha": 0.5,
    "reg_lambda": 0.5,
    "objective": "binary",
    "metric": "auc",
    "verbosity": -1,
    "n_jobs": -1,
    "random_state": 42,
}
lgb_model = lgb.LGBMClassifier(**best_params)

# Perform feature selection using RFECV
rfecv = RFECV(estimator=lgb_model, step=1, cv=KFold(10), scoring="roc_auc", n_jobs=-1)
rfecv.fit(X, y)

# Print the optimal number of features
print(f"Optimal number of features: {rfecv.n_features_}")

# Select the optimal features
X_selected = rfecv.transform(X)
X_test_selected = rfecv.transform(X_test)

# Retrain the model with the selected features
lgb_model.fit(X_selected, y)

# Predict on the test set with the selected features
final_predictions = lgb_model.predict_proba(X_test_selected)[:, 1]

# Save the submission file
submission = pd.DataFrame({"id": test_ids, "defects": final_predictions})
submission.to_csv("./working/submission.csv", index=False)

# Evaluate the model with selected features using cross-validation
auc_scores = []
kf = KFold(n_splits=10, shuffle=True, random_state=42)
for train_index, valid_index in kf.split(X_selected):
    X_train, X_valid = X_selected[train_index], X_selected[valid_index]
    y_train, y_valid = y.iloc[train_index], y.iloc[valid_index]
    lgb_model.fit(X_train, y_train)
    y_pred = lgb_model.predict_proba(X_valid)[:, 1]
    auc_score = roc_auc_score(y_valid, y_pred)
    auc_scores.append(auc_score)

# Print the mean AUC score
mean_auc_score = np.mean(auc_scores)
print(f"Mean AUC Score with Selected Features: {mean_auc_score}")



import pandas as pd
from sklearn.cluster import KMeans
from sklearn.metrics import silhouette_score
from sklearn.preprocessing import StandardScaler

# Load the dataset
data = pd.read_csv("./input/data.csv")
data.drop("id", axis=1, inplace=True)

# Standardize the features
scaler = StandardScaler()
data_scaled = scaler.fit_transform(data)

# Determine the optimal number of clusters using the elbow method and silhouette score
range_n_clusters = list(range(2, 11))
silhouette_scores = []
for n_clusters in range_n_clusters:
    clusterer = KMeans(n_clusters=n_clusters, random_state=42)
    cluster_labels = clusterer.fit_predict(data_scaled)
    silhouette_avg = silhouette_score(data_scaled, cluster_labels)
    silhouette_scores.append(silhouette_avg)

# Select the number of clusters with the highest silhouette score
optimal_n_clusters = range_n_clusters[silhouette_scores.index(max(silhouette_scores))]

# Fit the KMeans model with the optimal number of clusters
kmeans = KMeans(n_clusters=optimal_n_clusters, random_state=42)
data["Predicted"] = kmeans.fit_predict(data_scaled)

# Evaluate the model using silhouette score
final_silhouette_score = silhouette_score(data_scaled, data["Predicted"])
print(
    f"Silhouette Score for optimal clusters {optimal_n_clusters}: {final_silhouette_score}"
)

# Save the predictions to a CSV file
submission = pd.read_csv("./input/sample_submission.csv")
submission["Predicted"] = data["Predicted"]
submission.to_csv("./working/submission.csv", index=False)



import pandas as pd
from sklearn.ensemble import RandomForestClassifier
from sklearn.model_selection import train_test_split
from sklearn.metrics import accuracy_score

# Load the data
train_data = pd.read_csv("./input/train.csv")
test_data = pd.read_csv("./input/test.csv")

# Prepare the data
X = train_data.drop(["Id", "Cover_Type"], axis=1)
y = train_data["Cover_Type"]
X_train, X_val, y_train, y_val = train_test_split(X, y, test_size=0.2, random_state=42)

# Initialize the model
rf = RandomForestClassifier(n_estimators=100, random_state=42)

# Train the model
rf.fit(X_train, y_train)

# Validate the model
y_pred = rf.predict(X_val)
accuracy = accuracy_score(y_val, y_pred)
print(f"Validation Accuracy: {accuracy}")

# Predict on test data
test_ids = test_data["Id"]
test_data = test_data.drop("Id", axis=1)
test_predictions = rf.predict(test_data)

# Save the predictions
submission = pd.DataFrame({"Id": test_ids, "Cover_Type": test_predictions})
submission.to_csv("./working/submission.csv", index=False)



import pandas as pd
from sklearn.model_selection import train_test_split
from sklearn.linear_model import LogisticRegression
from sklearn.metrics import roc_auc_score

# Load the data
train_data = pd.read_csv("./input/train.csv")
test_data = pd.read_csv("./input/test.csv")

# Prepare the data
X = train_data.drop(["id", "booking_status"], axis=1)
y = train_data["booking_status"]
X_test = test_data.drop("id", axis=1)

# Split the data into training and validation sets
X_train, X_val, y_train, y_val = train_test_split(X, y, test_size=0.2, random_state=42)

# Initialize and train the logistic regression model
model = LogisticRegression(max_iter=1000, random_state=42)
model.fit(X_train, y_train)

# Predict on the validation set
val_predictions = model.predict_proba(X_val)[:, 1]
val_roc_auc = roc_auc_score(y_val, val_predictions)
print(f"Validation ROC AUC: {val_roc_auc}")

# Train the model on the full training data and predict on the test set
model.fit(X, y)
test_predictions = model.predict_proba(X_test)[:, 1]

# Save the predictions in the submission format
submission = pd.DataFrame({"id": test_data["id"], "booking_status": test_predictions})
submission.to_csv("./working/submission.csv", index=False)



import pandas as pd
from sklearn.ensemble import RandomForestClassifier
from sklearn.model_selection import train_test_split
from sklearn.multiclass import OneVsRestClassifier
from sklearn.preprocessing import LabelEncoder

# Load the data
train_data = pd.read_csv("./input/train.csv")
test_data = pd.read_csv("./input/test.csv")

# Prepare the data
X = train_data.drop(["id", "prognosis"], axis=1)
y = train_data["prognosis"]
X_test = test_data.drop(["id"], axis=1)
test_ids = test_data["id"]

# Encode the target variable
label_encoder = LabelEncoder()
y_encoded = label_encoder.fit_transform(y)

# Split the data into training and validation sets
X_train, X_val, y_train, y_val = train_test_split(
    X, y_encoded, test_size=0.2, random_state=42
)

# Train the model
model = OneVsRestClassifier(RandomForestClassifier(n_estimators=100, random_state=42))
model.fit(X_train, y_train)

# Predict on the validation set
y_val_pred_proba = model.predict_proba(X_val)

# Select the top 3 predictions for each sample
top3_preds = pd.DataFrame(y_val_pred_proba).apply(
    lambda x: label_encoder.inverse_transform(x.argsort()[-3:][::-1]), axis=1
)


# Evaluate the model using MPA@3
def mpa_at_k(y_true, y_pred, k=3):
    score = 0.0
    for true, pred in zip(y_true, y_pred):
        try:
            index = list(pred).index(true)
            score += 1.0 / (index + 1)
        except ValueError:
            continue
    return score / len(y_true)


# Calculate the MPA@3 score
y_val_true = label_encoder.inverse_transform(y_val)
mpa_score = mpa_at_k(y_val_true, top3_preds)
print(f"MPA@3 score on the validation set: {mpa_score}")

# Predict on the test set
y_test_pred_proba = model.predict_proba(X_test)
top3_test_preds = pd.DataFrame(y_test_pred_proba).apply(
    lambda x: label_encoder.inverse_transform(x.argsort()[-3:][::-1]), axis=1
)

# Prepare the submission file
submission = pd.DataFrame(
    {
        "id": test_ids,
        "prognosis": [" ".join(map(str, preds)) for preds in top3_test_preds],
    }
)
submission.to_csv("./working/submission.csv", index=False)



import pandas as pd
from sklearn.ensemble import RandomForestClassifier
from sklearn.model_selection import cross_val_score
from sklearn.impute import SimpleImputer
from sklearn.preprocessing import OneHotEncoder
from sklearn.compose import ColumnTransformer
from sklearn.pipeline import Pipeline

# Load the data
train_data = pd.read_csv("./input/train.csv")
test_data = pd.read_csv("./input/test.csv")

# Features and target
X = train_data.drop(["Survived", "PassengerId", "Name", "Ticket", "Cabin"], axis=1)
y = train_data["Survived"]

# Preprocessing for numerical data
numerical_transformer = SimpleImputer(strategy="median")

# Preprocessing for categorical data
categorical_cols = [cname for cname in X.columns if X[cname].dtype == "object"]
categorical_transformer = Pipeline(
    steps=[
        ("imputer", SimpleImputer(strategy="most_frequent")),
        ("onehot", OneHotEncoder(handle_unknown="ignore")),
    ]
)

# Bundle preprocessing for numerical and categorical data
preprocessor = ColumnTransformer(
    transformers=[
        ("num", numerical_transformer, ["Age", "Fare"]),
        ("cat", categorical_transformer, categorical_cols),
    ]
)

# Define the model
model = RandomForestClassifier(n_estimators=100, random_state=0)

# Bundle preprocessing and modeling code in a pipeline
clf = Pipeline(steps=[("preprocessor", preprocessor), ("model", model)])

# Cross-validation scores
scores = cross_val_score(clf, X, y, cv=10, scoring="accuracy")
print(f"Average cross-validation score: {scores.mean():.4f}")

# Preprocessing of test data, fit model
clf.fit(X, y)
test_X = test_data.drop(["PassengerId", "Name", "Ticket", "Cabin"], axis=1)
test_preds = clf.predict(test_X)

# Save test predictions to file
output = pd.DataFrame({"PassengerId": test_data.PassengerId, "Survived": test_preds})
output.to_csv("./working/submission.csv", index=False)



import pandas as pd
import lightgbm as lgb
from sklearn.model_selection import train_test_split
from sklearn.metrics import mean_squared_log_error
import numpy as np

# Load the data
train_data = pd.read_csv("./input/train.csv")
test_data = pd.read_csv("./input/test.csv")

# Prepare the data
X = train_data.drop(["id", "cost"], axis=1)
y = train_data["cost"]
X_test = test_data.drop("id", axis=1)

# Split the data into training and validation sets
X_train, X_val, y_train, y_val = train_test_split(X, y, test_size=0.2, random_state=42)

# Train the model
model = lgb.LGBMRegressor(random_state=42)
model.fit(X_train, y_train)

# Make predictions
y_pred = model.predict(X_val)
y_pred_test = model.predict(X_test)

# Calculate the RMSLE
rmsle = np.sqrt(mean_squared_log_error(y_val, y_pred))
print(f"Validation RMSLE: {rmsle}")

# Prepare the submission file
submission = pd.DataFrame({"id": test_data["id"], "cost": y_pred_test})
submission.to_csv("./working/submission.csv", index=False)



import pandas as pd
import lightgbm as lgb
from sklearn.model_selection import GridSearchCV, KFold
import numpy as np
from sklearn.metrics import make_scorer

# Load the data
train_data = pd.read_csv("./input/train.csv")
test_data = pd.read_csv("./input/test.csv")


# Preprocess the data
def preprocess_data(data):
    data["date"] = pd.to_datetime(data["date"])
    data["year"] = data["date"].dt.year
    data["month"] = data["date"].dt.month
    data["day"] = data["date"].dt.day
    data["dayofweek"] = data["date"].dt.dayofweek
    data["is_weekend"] = data["dayofweek"].apply(lambda x: 1 if x >= 5 else 0)
    data["week_of_year"] = data["date"].dt.isocalendar().week.astype(int)
    data["day_of_year"] = data["date"].dt.dayofyear
    data["sin_day_of_year"] = np.sin(2 * np.pi * data["day_of_year"] / 365.25)
    data["cos_day_of_year"] = np.cos(2 * np.pi * data["day_of_year"] / 365.25)
    data["country_month"] = data["country"] + "_" + data["month"].astype(str)
    data = pd.get_dummies(
        data, columns=["country", "store", "product", "country_month"]
    )
    return data


train_data = preprocess_data(train_data)
test_data = preprocess_data(test_data)

# Prepare the data for LightGBM
X = train_data.drop(["num_sold", "date", "row_id"], axis=1)
y = train_data["num_sold"]


# Define SMAPE function
def smape(y_true, y_pred):
    denominator = (np.abs(y_true) + np.abs(y_pred)) / 2.0
    diff = np.abs(y_true - y_pred) / denominator
    diff[denominator == 0] = 0.0
    return np.mean(diff)


# Custom scorer for cross-validation
smape_scorer = make_scorer(smape, greater_is_better=False)

# Define the parameter grid
param_grid = {
    "num_leaves": [31, 50, 70],
    "learning_rate": [0.1, 0.05, 0.01],
    "n_estimators": [100, 200, 500],
}

# Perform grid search with 10-fold cross-validation
gbm = lgb.LGBMRegressor(objective="regression", metric="mae", verbose=-1)
kf = KFold(n_splits=10, shuffle=True, random_state=42)
grid_search = GridSearchCV(
    estimator=gbm, param_grid=param_grid, cv=kf, scoring=smape_scorer, verbose=1
)
grid_search.fit(X, y)

# Train the model on full data with the best parameters
best_params = grid_search.best_params_
gbm_best = lgb.LGBMRegressor(**best_params)
gbm_best.fit(X, y)

# Predict on test set and save the submission file
X_test = test_data.drop(["date", "row_id"], axis=1)
test_data["num_sold"] = gbm_best.predict(X_test)
submission = test_data[["row_id", "num_sold"]]
submission.to_csv("./working/submission.csv", index=False)

# Print the evaluation metric
print("Best SMAPE (GridSearchCV):", -grid_search.best_score_)



import pandas as pd
import lightgbm as lgb
from sklearn.model_selection import RandomizedSearchCV, KFold
from sklearn.metrics import log_loss, make_scorer
from sklearn.preprocessing import LabelEncoder
import numpy as np

# Load the data
train_data = pd.read_csv("./input/train.csv")
test_data = pd.read_csv("./input/test.csv")

# Encode categorical features
le = LabelEncoder()
train_data["EJ"] = le.fit_transform(train_data["EJ"])
test_data["EJ"] = le.transform(test_data["EJ"])

# Prepare the data
X = train_data.drop(["Id", "Class"], axis=1)
y = train_data["Class"]
X_test = test_data.drop("Id", axis=1)

# Define the model parameters and parameter grid for randomized search
model = lgb.LGBMClassifier(objective="binary", boosting_type="gbdt", is_unbalance=True)
param_grid = {
    "learning_rate": [0.01, 0.05, 0.1],
    "num_leaves": [15, 31, 63],
    "max_depth": [-1, 5, 10],
    "min_child_samples": [10, 20, 30],
    "max_bin": [255, 300],
    "subsample": [0.6, 0.8, 1.0],
    "colsample_bytree": [0.3, 0.5, 0.7],
}

# Create a scorer for log loss
log_loss_scorer = make_scorer(log_loss, greater_is_better=False, needs_proba=True)

# Perform randomized search with cross-validation
random_search = RandomizedSearchCV(
    model,
    param_distributions=param_grid,
    n_iter=10,
    scoring=log_loss_scorer,
    cv=KFold(n_splits=10, shuffle=True, random_state=42),
    random_state=42,
    verbose=1,
)

random_search.fit(X, y)

# Best model and log loss
best_model = random_search.best_estimator_
best_score = -random_search.best_score_
print(f"Best Log Loss: {best_score}")

# Predict on test set with the best model
test_predictions = best_model.predict_proba(X_test)[:, 1]

# Create a submission file
submission = pd.DataFrame(
    {
        "Id": test_data["Id"],
        "class_0": 1 - test_predictions,
        "class_1": test_predictions,
    }
)
submission.to_csv("./working/submission.csv", index=False)



import pandas as pd
import numpy as np
import torch
import torch.nn as nn
import torch.nn.functional as F
import torch.optim as optim
from sklearn.model_selection import train_test_split
from torch.utils.data import DataLoader, TensorDataset
from torchvision import transforms

# Load the data
train_df = pd.read_csv("./input/train.csv")

# Prepare the data
X = train_df.drop("label", axis=1).values.reshape(-1, 1, 28, 28).astype("float32")
y = train_df["label"].values
X /= 255.0  # Normalize to [0, 1]

# Split into training and validation sets
X_train, X_val, y_train, y_val = train_test_split(X, y, test_size=0.2, random_state=42)

# Convert to PyTorch tensors
X_train_tensor = torch.tensor(X_train)
y_train_tensor = torch.tensor(y_train, dtype=torch.long)
X_val_tensor = torch.tensor(X_val)
y_val_tensor = torch.tensor(y_val, dtype=torch.long)

# Create datasets and dataloaders
train_dataset = TensorDataset(X_train_tensor, y_train_tensor)
val_dataset = TensorDataset(X_val_tensor, y_val_tensor)
train_loader = DataLoader(train_dataset, batch_size=64, shuffle=True)
val_loader = DataLoader(val_dataset, batch_size=64, shuffle=False)


# Define the CNN model
class Net(nn.Module):
    def __init__(self):
        super(Net, self).__init__()
        self.conv1 = nn.Conv2d(1, 32, kernel_size=3)
        self.conv2 = nn.Conv2d(32, 64, kernel_size=3)
        self.fc1 = nn.Linear(64 * 5 * 5, 128)
        self.fc2 = nn.Linear(128, 10)

    def forward(self, x):
        x = F.relu(F.max_pool2d(self.conv1(x), 2))
        x = F.relu(F.max_pool2d(self.conv2(x), 2))
        x = x.view(-1, 64 * 5 * 5)
        x = F.relu(self.fc1(x))
        x = self.fc2(x)
        return F.log_softmax(x, dim=1)


# Initialize the model, loss function, and optimizer
model = Net()
criterion = nn.CrossEntropyLoss()
optimizer = optim.Adam(model.parameters())

# Train the model
num_epochs = 5
for epoch in range(num_epochs):
    model.train()
    for data, target in train_loader:
        optimizer.zero_grad()
        output = model(data)
        loss = criterion(output, target)
        loss.backward()
        optimizer.step()

# Evaluate the model
model.eval()
correct = 0
with torch.no_grad():
    for data, target in val_loader:
        output = model(data)
        pred = output.argmax(dim=1, keepdim=True)
        correct += pred.eq(target.view_as(pred)).sum().item()

accuracy = correct / len(val_loader.dataset)
print(f"Validation Accuracy: {accuracy:.4f}")



import pandas as pd
import lightgbm as lgb
from sklearn.model_selection import train_test_split
from sklearn.metrics import log_loss

# Read dtypes and replace 'float16' with 'float32'
dtypes_df = pd.read_csv("./input/train_dtypes.csv")
dtypes = {
    k: (v if v != "float16" else "float32")
    for (k, v) in zip(dtypes_df.column, dtypes_df.dtype)
}

# Read and concatenate training data
train_dfs = [pd.read_csv(f"./input/train_{i}.csv", dtype=dtypes) for i in range(10)]
train_df = pd.concat(train_dfs, ignore_index=True)

# Prepare the data
X = train_df.drop(
    [
        "game_num",
        "event_id",
        "event_time",
        "player_scoring_next",
        "team_scoring_next",
        "team_A_scoring_within_10sec",
        "team_B_scoring_within_10sec",
    ],
    axis=1,
)
y = train_df[["team_A_scoring_within_10sec", "team_B_scoring_within_10sec"]]

# Split the data
X_train, X_val, y_train, y_val = train_test_split(X, y, test_size=0.2, random_state=42)

# Train the model for team A
model_A = lgb.LGBMClassifier()
model_A.fit(X_train, y_train.iloc[:, 0])

# Train the model for team B
model_B = lgb.LGBMClassifier()
model_B.fit(X_train, y_train.iloc[:, 1])

# Predict on validation set for team A
val_preds_A = model_A.predict_proba(X_val)[:, 1]

# Predict on validation set for team B
val_preds_B = model_B.predict_proba(X_val)[:, 1]

# Combine predictions
val_preds = pd.DataFrame(
    {
        "team_A_scoring_within_10sec": val_preds_A,
        "team_B_scoring_within_10sec": val_preds_B,
    }
)

# Calculate log loss
val_log_loss = log_loss(y_val, val_preds)
print(f"Validation Log Loss: {val_log_loss}")

# Predict on test set
test_dtypes_df = pd.read_csv("./input/test_dtypes.csv")
test_dtypes = {
    k: (v if v != "float16" else "float32")
    for (k, v) in zip(test_dtypes_df.column, test_dtypes_df.dtype)
}
test_df = pd.read_csv("./input/test.csv", dtype=test_dtypes)
X_test = test_df.drop(["id"], axis=1)

# Predict on test set for team A
test_preds_A = model_A.predict_proba(X_test)[:, 1]

# Predict on test set for team B
test_preds_B = model_B.predict_proba(X_test)[:, 1]

# Combine predictions
test_preds = pd.DataFrame(
    {
        "team_A_scoring_within_10sec": test_preds_A,
        "team_B_scoring_within_10sec": test_preds_B,
    }
)

# Prepare submission
submission = pd.concat([test_df["id"], test_preds], axis=1)
submission.to_csv("./working/submission.csv", index=False)



import lightgbm as lgb
import pandas as pd
import numpy as np
from sklearn.model_selection import train_test_split
from sklearn.metrics import mean_squared_log_error
from sklearn.preprocessing import OneHotEncoder

# Load the data
train_df = pd.read_csv("./input/train.csv")
test_df = pd.read_csv("./input/test.csv")

# Feature engineering on 'release_date'
train_df["release_date"] = pd.to_datetime(train_df["release_date"])
test_df["release_date"] = pd.to_datetime(test_df["release_date"], errors="coerce")

train_df["release_year"] = train_df["release_date"].dt.year
train_df["release_month"] = train_df["release_date"].dt.month
train_df["release_dayofweek"] = train_df["release_date"].dt.dayofweek

test_df["release_year"] = test_df["release_date"].dt.year
test_df["release_month"] = test_df["release_date"].dt.month
test_df["release_dayofweek"] = test_df["release_date"].dt.dayofweek

# Fill missing 'release_date' derived features in test set with median from train set
test_df["release_year"] = test_df["release_year"].fillna(
    train_df["release_year"].median()
)
test_df["release_month"] = test_df["release_month"].fillna(
    train_df["release_month"].median()
)
test_df["release_dayofweek"] = test_df["release_dayofweek"].fillna(
    train_df["release_dayofweek"].median()
)

# Select features for the model
features = [
    "budget",
    "popularity",
    "runtime",
    "original_language",
    "release_year",
    "release_month",
    "release_dayofweek",
]
target = "revenue"

# Process categorical features with one-hot encoding
categorical_features = ["original_language"]
encoder = OneHotEncoder(handle_unknown="ignore")
encoded_features = encoder.fit_transform(train_df[categorical_features]).toarray()
encoded_feature_names = encoder.get_feature_names_out(categorical_features)
encoded_df = pd.DataFrame(encoded_features, columns=encoded_feature_names)

# Fill missing numerical feature values with median
for feature in features:
    if feature not in categorical_features:
        train_df[feature] = train_df[feature].fillna(train_df[feature].median())

# Combine numerical and encoded categorical features
X = pd.concat(
    [train_df[features].drop(columns=categorical_features), encoded_df], axis=1
)
y = np.log1p(train_df[target])

# Split the data into training and validation sets
X_train, X_valid, y_train, y_valid = train_test_split(
    X, y, test_size=0.2, random_state=42
)

# Train the LightGBM model
model = lgb.LGBMRegressor()
model.fit(X_train, y_train)

# Predict on the validation set
y_pred = model.predict(X_valid)

# Calculate the RMSLE
rmsle = np.sqrt(mean_squared_log_error(y_valid, y_pred))
print(f"RMSLE: {rmsle}")

# Prepare the test set
test_encoded_features = encoder.transform(test_df[categorical_features]).toarray()
test_encoded_df = pd.DataFrame(test_encoded_features, columns=encoded_feature_names)
# Fill missing numerical feature values with median in the test set
for feature in features:
    if feature not in categorical_features:
        test_df[feature] = test_df[feature].fillna(train_df[feature].median())
X_test = pd.concat(
    [test_df[features].drop(columns=categorical_features), test_encoded_df], axis=1
)

# Predict on the test set
test_pred = model.predict(X_test)

# Prepare the submission file
submission = pd.DataFrame({"id": test_df["id"], "revenue": np.expm1(test_pred)})
submission.to_csv("./working/submission.csv", index=False)



from dataclasses import dataclass

import jsonschema
from dataclasses_json import DataClassJsonMixin

PromptType = str | dict | list
FunctionCallType = dict
OutputType = str | FunctionCallType


def opt_messages_to_list(
    system_message: str | None, user_message: str | None
) -> list[dict[str, str]]:
    messages = []
    if system_message:
        messages.append({"role": "system", "content": system_message})
    if user_message:
        messages.append({"role": "user", "content": user_message})
    return messages


def compile_prompt_to_md(prompt: PromptType, _header_depth: int = 1) -> str:
    if isinstance(prompt, str):
        return prompt.strip() + "\n"
    elif isinstance(prompt, list):
        return "\n".join([f"- {s.strip()}" for s in prompt] + ["\n"])

    out = []
    header_prefix = "#" * _header_depth
    for k, v in prompt.items():
        out.append(f"{header_prefix} {k}\n")
        out.append(compile_prompt_to_md(v, _header_depth=_header_depth + 1))
    return "\n".join(out)


@dataclass
class FunctionSpec(DataClassJsonMixin):
    name: str
    json_schema: dict  # JSON schema
    description: str

    def __post_init__(self):
        # validate the schema
        jsonschema.Draft7Validator.check_schema(self.json_schema)

    @property
    def as_openai_tool_dict(self):
        return {
            "type": "function",
            "function": {
                "name": self.name,
                "description": self.description,
                "parameters": self.json_schema,
            },
        }

    @property
    def openai_tool_choice_dict(self):
        return {
            "type": "function",
            "function": {"name": self.name},
        }



import pandas as pd
from sklearn.model_selection import train_test_split
from sklearn.metrics import roc_auc_score
from lightgbm import LGBMClassifier
from sklearn.preprocessing import OneHotEncoder

# Load the data
train_data = pd.read_csv("./input/train.csv")
test_data = pd.read_csv("./input/test.csv")

# Separate target from predictors
y = train_data.target
X = train_data.drop(["target", "id"], axis=1)
X_test = test_data.drop(["id"], axis=1)

# One-hot encode the categorical data
cat_cols = [col for col in X.columns if "cat" in col]
encoder = OneHotEncoder(handle_unknown="ignore", sparse=False)
X_encoded = pd.DataFrame(encoder.fit_transform(X[cat_cols]))
X_test_encoded = pd.DataFrame(encoder.transform(X_test[cat_cols]))

# One-hot encoding removed index; put it back
X_encoded.index = X.index
X_test_encoded.index = X_test.index

# Remove categorical columns (will replace with one-hot encoding)
num_X = X.drop(cat_cols, axis=1)
num_X_test = X_test.drop(cat_cols, axis=1)

# Add one-hot encoded columns to numerical features
X_final = pd.concat([num_X, X_encoded], axis=1)
X_test_final = pd.concat([num_X_test, X_test_encoded], axis=1)

# Split the data into training and validation sets
X_train, X_valid, y_train, y_valid = train_test_split(
    X_final, y, train_size=0.8, test_size=0.2, random_state=0
)

# Define the model
model = LGBMClassifier()

# Train the model
model.fit(X_train, y_train)

# Predict on the validation set
val_predictions = model.predict_proba(X_valid)[:, 1]

# Calculate the AUC score
val_auc = roc_auc_score(y_valid, val_predictions)

# Print the AUC score
print(f"Validation AUC: {val_auc}")

# Predict on the test set
test_predictions = model.predict_proba(X_test_final)[:, 1]

# Save the predictions to a CSV file
output = pd.DataFrame({"id": test_data.id, "target": test_predictions})
output.to_csv("./working/submission.csv", index=False)



"""
Contains functions to manually generate a textual preview of some common file types (.csv, .json,..) for the agent.
"""

import json
from pathlib import Path

import humanize
import pandas as pd
from genson import SchemaBuilder
from pandas.api.types import is_numeric_dtype

# these files are treated as code (e.g. markdown wrapped)
code_files = {".py", ".sh", ".yaml", ".yml", ".md", ".html", ".xml", ".log", ".rst"}
# we treat these files as text (rather than binary) files
plaintext_files = {".txt", ".csv", ".json", ".tsv"} | code_files


def get_file_len_size(f: Path) -> tuple[int, str]:
    """
    Calculate the size of a file (#lines for plaintext files, otherwise #bytes)
    Also returns a human-readable string representation of the size.
    """
    if f.suffix in plaintext_files:
        num_lines = sum(1 for _ in open(f))
        return num_lines, f"{num_lines} lines"
    else:
        s = f.stat().st_size
        return s, humanize.naturalsize(s)


def file_tree(path: Path, depth=0) -> str:
    """Generate a tree structure of files in a directory"""
    result = []
    files = [p for p in Path(path).iterdir() if not p.is_dir()]
    dirs = [p for p in Path(path).iterdir() if p.is_dir()]
    max_n = 4 if len(files) > 30 else 8
    for p in sorted(files)[:max_n]:
        result.append(f"{' '*depth*4}{p.name} ({get_file_len_size(p)[1]})")
    if len(files) > max_n:
        result.append(f"{' '*depth*4}... and {len(files)-max_n} other files")

    for p in sorted(dirs):
        result.append(f"{' '*depth*4}{p.name}/")
        result.append(file_tree(p, depth + 1))

    return "\n".join(result)


def _walk(path: Path):
    """Recursively walk a directory (analogous to os.walk but for pathlib.Path)"""
    for p in sorted(Path(path).iterdir()):
        if p.is_dir():
            yield from _walk(p)
            continue
        yield p


def preview_csv(p: Path, file_name: str, simple=True) -> str:
    """Generate a textual preview of a csv file

    Args:
        p (Path): the path to the csv file
        file_name (str): the file name to use in the preview
        simple (bool, optional): whether to use a simplified version of the preview. Defaults to True.

    Returns:
        str: the textual preview
    """
    df = pd.read_csv(p)

    out = []

    out.append(f"-> {file_name} has {df.shape[0]} rows and {df.shape[1]} columns.")

    if simple:
        cols = df.columns.tolist()
        sel_cols = 15
        cols_str = ", ".join(cols[:sel_cols])
        res = f"The columns are: {cols_str}"
        if len(cols) > sel_cols:
            res += f"... and {len(cols)-sel_cols} more columns"
        out.append(res)
    else:
        out.append("Here is some information about the columns:")
        for col in sorted(df.columns):
            dtype = df[col].dtype
            name = f"{col} ({dtype})"

            nan_count = df[col].isnull().sum()

            if dtype == "bool":
                v = df[col][df[col].notnull()].mean()
                out.append(f"{name} is {v*100:.2f}% True, {100-v*100:.2f}% False")
            elif df[col].nunique() < 10:
                out.append(
                    f"{name} has {df[col].nunique()} unique values: {df[col].unique().tolist()}"
                )
            elif is_numeric_dtype(df[col]):
                out.append(
                    f"{name} has range: {df[col].min():.2f} - {df[col].max():.2f}, {nan_count} nan values"
                )
            elif dtype == "object":
                out.append(
                    f"{name} has {df[col].nunique()} unique values. Some example values: {df[col].value_counts().head(4).index.tolist()}"
                )

    return "\n".join(out)


def preview_json(p: Path, file_name: str):
    """Generate a textual preview of a json file using a generated json schema"""
    builder = SchemaBuilder()
    with open(p) as f:
        builder.add_object(json.load(f))
    return f"-> {file_name} has auto-generated json schema:\n" + builder.to_json(
        indent=2
    )


def generate(base_path, include_file_details=True, simple=False):
    """
    Generate a textual preview of a directory, including an overview of the directory
    structure and previews of individual files
    """
    tree = f"```\n{file_tree(base_path)}```"
    out = [tree]

    if include_file_details:
        for fn in _walk(base_path):
            file_name = str(fn.relative_to(base_path))

            if fn.suffix == ".csv":
                out.append(preview_csv(fn, file_name, simple=simple))
            elif fn.suffix == ".json":
                out.append(preview_json(fn, file_name))
            elif fn.suffix in plaintext_files:
                if get_file_len_size(fn)[0] < 30:
                    with open(fn) as f:
                        content = f.read()
                        if fn.suffix in code_files:
                            content = f"```\n{content}\n```"
                        out.append(f"-> {file_name} has content:\n\n{content}")

    result = "\n\n".join(out)

    # if the result is very long we generate a simpler version
    if len(result) > 6_000 and not simple:
        return generate(
            base_path, include_file_details=include_file_details, simple=True
        )

    return result



import pandas as pd
from sklearn.feature_extraction.text import TfidfVectorizer
from sklearn.linear_model import LogisticRegression
from sklearn.model_selection import train_test_split
from sklearn.metrics import f1_score

# Load the data
train_data = pd.read_csv("./input/train.csv")
test_data = pd.read_csv("./input/test.csv")

# Prepare the data
X_train, X_val, y_train, y_val = train_test_split(
    train_data["text"], train_data["target"], test_size=0.2, random_state=42
)

# Vectorize the text data
vectorizer = TfidfVectorizer()
X_train_tfidf = vectorizer.fit_transform(X_train)
X_val_tfidf = vectorizer.transform(X_val)

# Train the logistic regression model
model = LogisticRegression()
model.fit(X_train_tfidf, y_train)

# Predict on the validation set
val_predictions = model.predict(X_val_tfidf)

# Evaluate the model
f1 = f1_score(y_val, val_predictions)
print(f"F1 Score on the validation set: {f1}")

# Predict on the test set and save the submission
X_test_tfidf = vectorizer.transform(test_data["text"])
test_predictions = model.predict(X_test_tfidf)
submission = pd.DataFrame({"id": test_data["id"], "target": test_predictions})
submission.to_csv("./working/submission.csv", index=False)



import pandas as pd
import lightgbm as lgb
from sklearn.model_selection import train_test_split
from sklearn.metrics import log_loss

# Load the data
train_data = pd.read_csv("./input/train.csv")
test_data = pd.read_csv("./input/test.csv")

# Prepare the data
X = train_data.drop(["id", "target"], axis=1)
y = train_data["target"].astype("category")
X_test = test_data.drop(["id"], axis=1)

# Split the data
X_train, X_val, y_train, y_val = train_test_split(X, y, test_size=0.2, random_state=42)

# Create and train the model
model = lgb.LGBMClassifier(objective="multiclass", random_state=42)
model.fit(X_train, y_train)

# Evaluate the model
val_preds = model.predict_proba(X_val)
print(f"Validation Log Loss: {log_loss(y_val, val_preds)}")

# Predict on test set
test_preds = model.predict_proba(X_test)

# Prepare the submission file
submission = pd.DataFrame(
    test_preds, columns=["Class_1", "Class_2", "Class_3", "Class_4"]
)
submission["id"] = test_data["id"]
submission = submission[["id", "Class_1", "Class_2", "Class_3", "Class_4"]]
submission.to_csv("./working/submission.csv", index=False)



import pandas as pd
import numpy as np
from sklearn.model_selection import train_test_split
from sklearn.metrics import cohen_kappa_score
import lightgbm as lgb

# Load the data
train_data = pd.read_csv("./input/train.csv")
test_data = pd.read_csv("./input/test.csv")

# Prepare the data
X = train_data.drop(["Id", "quality"], axis=1)
y = train_data["quality"]
X_test = test_data.drop("Id", axis=1)

# Split the data into training and validation sets
X_train, X_val, y_train, y_val = train_test_split(X, y, test_size=0.2, random_state=42)

# Train the model
model = lgb.LGBMRegressor(random_state=42)
model.fit(X_train, y_train)

# Make predictions
val_predictions = model.predict(X_val)
val_predictions = np.round(val_predictions).astype(int)  # Round to the nearest integer

# Evaluate the model
kappa_score = cohen_kappa_score(y_val, val_predictions, weights="quadratic")
print(f"Quadratic Weighted Kappa score on validation set: {kappa_score}")

# Make predictions on the test set
test_predictions = model.predict(X_test)
test_predictions = np.round(test_predictions).astype(
    int
)  # Round to the nearest integer

# Prepare the submission file
submission = pd.DataFrame({"Id": test_data["Id"], "quality": test_predictions})
submission.to_csv("./working/submission.csv", index=False)



import pandas as pd
from sklearn.ensemble import RandomForestRegressor
from sklearn.model_selection import train_test_split
from sklearn.metrics import mean_squared_error
from math import sqrt

# Load the data
games = pd.read_csv("./input/games.csv")
turns = pd.read_csv("./input/turns.csv")
train = pd.read_csv("./input/train.csv")

# Merge the datasets on game_id
merged_data = pd.merge(train, games, on="game_id")
merged_data = pd.merge(
    merged_data,
    turns.groupby("game_id").agg({"points": "sum"}).reset_index(),
    on="game_id",
)

# Prepare the features and target variable
X = merged_data[["game_duration_seconds", "winner", "points"]]
y = merged_data["rating"]

# Split the data into training and validation sets
X_train, X_val, y_train, y_val = train_test_split(X, y, test_size=0.2, random_state=42)

# Initialize the model
model = RandomForestRegressor(n_estimators=100, random_state=42)

# Train the model
model.fit(X_train, y_train)

# Predict on the validation set
y_pred = model.predict(X_val)

# Calculate the RMSE
rmse = sqrt(mean_squared_error(y_val, y_pred))
print(f"Validation RMSE: {rmse}")

# Prepare the test set
test = pd.read_csv("./input/test.csv")
test_merged = pd.merge(test, games, on="game_id")
test_merged = pd.merge(
    test_merged,
    turns.groupby("game_id").agg({"points": "sum"}).reset_index(),
    on="game_id",
)
X_test = test_merged[["game_duration_seconds", "winner", "points"]]

# Predict on the test set
test["rating"] = model.predict(X_test)

# Save the predictions to a CSV file
test[["game_id", "rating"]].to_csv("./working/submission.csv", index=False)



import copy
import json
from pathlib import Path
from typing import Type, TypeVar

import dataclasses_json
from ..journal import Journal


def dumps_json(obj: dataclasses_json.DataClassJsonMixin):
    """Serialize AIDE dataclasses (such as Journals) to JSON."""
    if isinstance(obj, Journal):
        obj = copy.deepcopy(obj)
        node2parent = {n.id: n.parent.id for n in obj.nodes if n.parent is not None}
        for n in obj.nodes:
            n.parent = None
            n.children = set()

    obj_dict = obj.to_dict()

    if isinstance(obj, Journal):
        obj_dict["node2parent"] = node2parent  # type: ignore
        obj_dict["__version"] = "2"

    return json.dumps(obj_dict, separators=(",", ":"))


def dump_json(obj: dataclasses_json.DataClassJsonMixin, path: Path):
    with open(path, "w") as f:
        f.write(dumps_json(obj))


G = TypeVar("G", bound=dataclasses_json.DataClassJsonMixin)


def loads_json(s: str, cls: Type[G]) -> G:
    """Deserialize JSON to AIDE dataclasses."""
    obj_dict = json.loads(s)
    obj = cls.from_dict(obj_dict)

    if isinstance(obj, Journal):
        id2nodes = {n.id: n for n in obj.nodes}
        for child_id, parent_id in obj_dict["node2parent"].items():
            id2nodes[child_id].parent = id2nodes[parent_id]
            id2nodes[child_id].__post_init__()
    return obj


def load_json(path: Path, cls: Type[G]) -> G:
    with open(path, "r") as f:
        return loads_json(f.read(), cls)



import pandas as pd
import lightgbm as lgb
from sklearn.metrics import mean_squared_error
from sklearn.model_selection import train_test_split

# Load the data
train_data = pd.read_csv("./input/train.csv")
test_data = pd.read_csv("./input/test.csv")

# Preprocess the data
train_data[["ID", "latitude", "longitude", "year", "week_no"]] = train_data[
    "ID_LAT_LON_YEAR_WEEK"
].str.split("_", expand=True)
test_data[["ID", "latitude", "longitude", "year", "week_no"]] = test_data[
    "ID_LAT_LON_YEAR_WEEK"
].str.split("_", expand=True)

# Convert to numeric types
for col in ["latitude", "longitude", "year", "week_no"]:
    train_data[col] = pd.to_numeric(train_data[col])
    test_data[col] = pd.to_numeric(test_data[col])

# One-hot encoding for 'week_no'
train_data = pd.get_dummies(train_data, columns=["week_no"])
test_data = pd.get_dummies(test_data, columns=["week_no"])

# Align test_data columns with train_data
test_data = test_data.reindex(columns=train_data.columns, fill_value=0)

# Prepare the data for training
X = train_data.drop(columns=["emission", "ID_LAT_LON_YEAR_WEEK", "ID"])
y = train_data["emission"]
X_train, X_val, y_train, y_val = train_test_split(X, y, test_size=0.2, random_state=42)

# Train the LightGBM model
lgb_train = lgb.Dataset(X_train, y_train)
lgb_eval = lgb.Dataset(X_val, y_val, reference=lgb_train)

params = {"objective": "regression", "metric": "rmse", "verbose": -1}

gbm = lgb.train(
    params,
    lgb_train,
    num_boost_round=100,
    valid_sets=lgb_eval,
    early_stopping_rounds=10,
)

# Predict on validation set
y_pred = gbm.predict(X_val, num_iteration=gbm.best_iteration)

# Evaluate the model
rmse = mean_squared_error(y_val, y_pred, squared=False)
print(f"Validation RMSE: {rmse}")

# Predict on test set and save submission
test_features = test_data.drop(columns=["ID_LAT_LON_YEAR_WEEK", "ID", "emission"])
test_data["emission"] = gbm.predict(test_features, num_iteration=gbm.best_iteration)
submission = test_data[["ID_LAT_LON_YEAR_WEEK", "emission"]]
submission.to_csv("./working/submission.csv", index=False)



"""Export journal to HTML visualization of tree + code."""

import json
import textwrap
from pathlib import Path

import numpy as np
from igraph import Graph
from ..journal import Journal


def get_edges(journal: Journal):
    for node in journal:
        for c in node.children:
            yield (node.step, c.step)


def generate_layout(n_nodes, edges, layout_type="rt"):
    """Generate visual layout of graph"""
    layout = Graph(
        n_nodes,
        edges=edges,
        directed=True,
    ).layout(layout_type)
    y_max = max(layout[k][1] for k in range(n_nodes))
    layout_coords = []
    for n in range(n_nodes):
        layout_coords.append((layout[n][0], 2 * y_max - layout[n][1]))
    return np.array(layout_coords)


def normalize_layout(layout: np.ndarray):
    """Normalize layout to [0, 1]"""
    layout = (layout - layout.min(axis=0)) / (layout.max(axis=0) - layout.min(axis=0))
    layout[:, 1] = 1 - layout[:, 1]
    layout[:, 1] = np.nan_to_num(layout[:, 1], nan=0)
    layout[:, 0] = np.nan_to_num(layout[:, 0], nan=0.5)
    return layout


def cfg_to_tree_struct(cfg, jou: Journal):
    edges = list(get_edges(jou))
    layout = normalize_layout(generate_layout(len(jou), edges))

    # metrics = np.array([n.metric.value_npsafe for n in jou])
    # metrics = (metrics - np.nanmin(metrics)) / (np.nanmax(metrics) - np.nanmin(metrics))
    # metrics = np.nan_to_num(metrics, nan=1)
    # metrics[:] = 0
    metrics = np.array([0 for n in jou])

    return dict(
        edges=edges,
        layout=layout.tolist(),
        plan=[textwrap.fill(n.plan, width=80) for n in jou.nodes],
        code=[n.code for n in jou],
        term_out=[n.term_out for n in jou],
        analysis=[n.analysis for n in jou],
        exp_name=cfg.exp_name,
        metrics=metrics.tolist(),
    )


def generate_html(tree_graph_str: str):
    template_dir = Path(__file__).parent / "viz_templates"
    
    with open(template_dir / "template.js") as f:
        js = f.read()
        js = js.replace("<placeholder>", tree_graph_str)

    with open(template_dir / "template.html") as f:
        html = f.read()
        html = html.replace("<!-- placeholder -->", js)

        return html


def generate(cfg, jou: Journal, out_path: Path):
    tree_graph_str = json.dumps(cfg_to_tree_struct(cfg, jou))
    html = generate_html(tree_graph_str)
    with open(out_path, "w") as f:
        f.write(html)



import pandas as pd
from sklearn.ensemble import RandomForestClassifier
from sklearn.metrics import roc_auc_score
from sklearn.model_selection import train_test_split

# Load the data
train_data = pd.read_csv("./input/train.csv")
train_labels = pd.read_csv("./input/train_labels.csv")
test_data = pd.read_csv("./input/test.csv")

# Aggregate features for each sequence
agg_funcs = ["mean", "std", "min", "max"]
train_features = train_data.groupby("sequence").agg(agg_funcs)
test_features = test_data.groupby("sequence").agg(agg_funcs)

# Flatten multi-level columns
train_features.columns = [
    "_".join(col).strip() for col in train_features.columns.values
]
test_features.columns = ["_".join(col).strip() for col in test_features.columns.values]

# Split the data into train and validation sets
X_train, X_val, y_train, y_val = train_test_split(
    train_features, train_labels["state"], test_size=0.2, random_state=42
)

# Initialize and train the Random Forest classifier
rf = RandomForestClassifier(n_estimators=100, random_state=42)
rf.fit(X_train, y_train)

# Predict probabilities for the validation set
val_probs = rf.predict_proba(X_val)[:, 1]

# Calculate the AUC-ROC score
auc_score = roc_auc_score(y_val, val_probs)
print(f"AUC-ROC score: {auc_score}")

# Predict probabilities for the test set
test_probs = rf.predict_proba(test_features)[:, 1]

# Create the submission file
submission = pd.DataFrame({"sequence": test_features.index, "state": test_probs})
submission.to_csv("./working/submission.csv", index=False)



from . import backend_anthropic, backend_openai
from .utils import FunctionSpec, OutputType, PromptType, compile_prompt_to_md


def query(
    system_message: PromptType | None,
    user_message: PromptType | None,
    model: str,
    temperature: float | None = None,
    max_tokens: int | None = None,
    func_spec: FunctionSpec | None = None,
    **model_kwargs,
) -> OutputType:
    """
    General LLM query for various backends with a single system and user message.
    Supports function calling for some backends.

    Args:
        system_message (PromptType | None): Uncompiled system message (will generate a message following the OpenAI/Anthropic format)
        user_message (PromptType | None): Uncompiled user message (will generate a message following the OpenAI/Anthropic format)
        model (str): string identifier for the model to use (e.g. "gpt-4-turbo")
        temperature (float | None, optional): Temperature to sample at. Defaults to the model-specific default.
        max_tokens (int | None, optional): Maximum number of tokens to generate. Defaults to the model-specific max tokens.
        func_spec (FunctionSpec | None, optional): Optional FunctionSpec object defining a function call. If given, the return value will be a dict.

    Returns:
        OutputType: A string completion if func_spec is None, otherwise a dict with the function call details.
    """

    model_kwargs = model_kwargs | {
        "model": model,
        "temperature": temperature,
        "max_tokens": max_tokens,
    }

    query_func = backend_openai.query if "gpt-" in model else backend_anthropic.query
    output, req_time, in_tok_count, out_tok_count, info = query_func(
        system_message=compile_prompt_to_md(system_message) if system_message else None,
        user_message=compile_prompt_to_md(user_message) if user_message else None,
        func_spec=func_spec,
        **model_kwargs,
    )

    return output



import pandas as pd
import numpy as np
import lightgbm as lgb
from sklearn.model_selection import train_test_split
from sklearn.metrics import accuracy_score
from sklearn.preprocessing import LabelEncoder

# Load the data
train_data = pd.read_csv("./input/train.csv")
test_data = pd.read_csv("./input/test.csv")

# Encode the 'target' column
le = LabelEncoder()
train_data["target"] = le.fit_transform(train_data["target"])

# Separate features and target
X = train_data.drop(["row_id", "target"], axis=1)
y = train_data["target"]

# Split the data into training and validation sets
X_train, X_val, y_train, y_val = train_test_split(X, y, test_size=0.2, random_state=42)

# Create LightGBM datasets
train_set = lgb.Dataset(X_train, label=y_train)
val_set = lgb.Dataset(X_val, label=y_val)

# Adjusted parameters for LightGBM
params = {
    "objective": "multiclass",
    "num_class": len(le.classes_),
    "metric": "multi_logloss",
    "boosting_type": "gbdt",
    "learning_rate": 0.05,
    "num_leaves": 50,
    "max_depth": -1,
    "min_data_in_leaf": 20,
    "feature_fraction": 0.75,  # Adjusted feature fraction
    "bagging_fraction": 0.85,  # Adjusted bagging fraction
    "bagging_freq": 5,
}

# Train the model
gbm = lgb.train(
    params,
    train_set,
    num_boost_round=1000,
    valid_sets=[train_set, val_set],
    early_stopping_rounds=100,
    verbose_eval=100,
)

# Predict on validation set
y_pred = gbm.predict(X_val, num_iteration=gbm.best_iteration)
y_pred_max = [np.argmax(line) for line in y_pred]

# Evaluate the model
accuracy = accuracy_score(y_val, y_pred_max)
print(f"Validation Accuracy: {accuracy}")

# Predict on test set
X_test = test_data.drop(["row_id"], axis=1)
test_pred = gbm.predict(X_test, num_iteration=gbm.best_iteration)
test_pred_max = [np.argmax(line) for line in test_pred]

# Inverse transform the predicted labels
test_pred_labels = le.inverse_transform(test_pred_max)

# Prepare submission
submission = pd.DataFrame({"row_id": test_data["row_id"], "target": test_pred_labels})
submission.to_csv("./working/submission.csv", index=False)



import pandas as pd
from sklearn.linear_model import LinearRegression
from sklearn.model_selection import cross_val_score, KFold
from sklearn.impute import SimpleImputer
import numpy as np

# Load the data
data = pd.read_csv("./input/data.csv")

# Select only numeric columns for imputation
numeric_columns = data.select_dtypes(include=[np.number]).columns
X = data[numeric_columns].drop(columns=["x_e_out [-]"])
y = data["x_e_out [-]"]

# Split the data into training and NaN sets
train_data = data.dropna(subset=["x_e_out [-]"])
nan_data = data[data["x_e_out [-]"].isna()]

# Impute missing values in features with mean
imputer = SimpleImputer(strategy="mean")
X_train_imputed = imputer.fit_transform(
    train_data[numeric_columns].drop(columns=["x_e_out [-]"])
)
y_train = train_data["x_e_out [-]"]

# Train the linear regression model using cross-validation
model = LinearRegression()
kf = KFold(n_splits=10, shuffle=True, random_state=1)
rmse_scores = cross_val_score(
    model, X_train_imputed, y_train, scoring="neg_root_mean_squared_error", cv=kf
)
print(f"10-fold CV RMSE: {-np.mean(rmse_scores):.4f} (+/- {np.std(rmse_scores):.4f})")

# Fit the model on the entire training set
model.fit(X_train_imputed, y_train)

# Load the test data
test_data = pd.read_csv("./input/sample_submission.csv")

# Prepare the test features
X_test = nan_data[numeric_columns].drop(columns=["x_e_out [-]"])
X_test_imputed = imputer.transform(X_test)

# Predict the missing values for the test set
nan_data["x_e_out [-]"] = model.predict(X_test_imputed)

# Merge predictions back into the test data
test_data = test_data.merge(nan_data[["id", "x_e_out [-]"]], on="id", how="left")

# Save the submission file
test_data.to_csv("./working/submission.csv", index=False)



import pandas as pd
from catboost import CatBoostRegressor
from sklearn.model_selection import train_test_split
from sklearn.metrics import mean_squared_error
import numpy as np

# Load the data
train_data = pd.read_csv("./input/train.csv")
test_data = pd.read_csv("./input/test.csv")

# Separate features and target
X = train_data.drop(["id", "target"], axis=1)
y = train_data["target"]

# Split data into training and validation sets
X_train, X_val, y_train, y_val = train_test_split(X, y, test_size=0.2, random_state=42)

# Identify categorical features
cat_features = [col for col in X_train.columns if X_train[col].dtype == "object"]

# Define hyperparameter space
learning_rates = [0.03, 0.1]
depths = [4, 6, 8]
n_estimators = [100, 500, 1000]

best_rmse = float("inf")
best_params = {}

# Grid search
for learning_rate in learning_rates:
    for depth in depths:
        for n_estimator in n_estimators:
            model = CatBoostRegressor(
                loss_function="RMSE",
                cat_features=cat_features,
                verbose=200,
                random_seed=42,
                learning_rate=learning_rate,
                depth=depth,
                n_estimators=n_estimator,
            )
            model.fit(
                X_train,
                y_train,
                eval_set=(X_val, y_val),
                early_stopping_rounds=50,
                use_best_model=True,
            )
            y_pred = model.predict(X_val)
            rmse = np.sqrt(mean_squared_error(y_val, y_pred))
            if rmse < best_rmse:
                best_rmse = rmse
                best_params = {
                    "learning_rate": learning_rate,
                    "depth": depth,
                    "n_estimators": n_estimator,
                }

# Train the model with best parameters
model = CatBoostRegressor(
    loss_function="RMSE",
    cat_features=cat_features,
    verbose=200,
    random_seed=42,
    **best_params,
)
model.fit(X, y)

# Predict on test data
test_predictions = model.predict(test_data.drop(["id"], axis=1))

# Save test predictions to file
submission = pd.DataFrame({"id": test_data["id"], "target": test_predictions})
submission.to_csv("./working/submission.csv", index=False)

print(f"Best Validation RMSE: {best_rmse}")
print(f"Best Parameters: {best_params}")



import pandas as pd
from sklearn.model_selection import train_test_split
from sklearn.linear_model import LogisticRegression
from sklearn.metrics import roc_auc_score
from sklearn.preprocessing import OneHotEncoder
from sklearn.impute import SimpleImputer

# Load the data
train_data = pd.read_csv("./input/train.csv")
test_data = pd.read_csv("./input/test.csv")

# Preprocess the data
features = train_data.columns.drop(["id", "failure"])
X = train_data[features]
y = train_data["failure"]
X_test = test_data[features]

# Fill missing values with median for numerical columns
num_cols = X.select_dtypes(exclude="object").columns
imputer = SimpleImputer(strategy="median")
X[num_cols] = imputer.fit_transform(X[num_cols])
X_test[num_cols] = imputer.transform(X_test[num_cols])

# One-hot encode categorical features
cat_cols = X.select_dtypes(include="object").columns
encoder = OneHotEncoder(handle_unknown="ignore", sparse=False)
X_encoded = pd.DataFrame(
    encoder.fit_transform(X[cat_cols]), columns=encoder.get_feature_names_out(cat_cols)
)
X_test_encoded = pd.DataFrame(
    encoder.transform(X_test[cat_cols]), columns=encoder.get_feature_names_out(cat_cols)
)

# One-hot encoding removed index; put it back
X_encoded.index = X.index
X_test_encoded.index = X_test.index

# Remove categorical columns (will replace with one-hot encoding)
num_X = X.drop(cat_cols, axis=1)
num_X_test = X_test.drop(cat_cols, axis=1)

# Add one-hot encoded columns to numerical features
X_preprocessed = pd.concat([num_X, X_encoded], axis=1)
X_test_preprocessed = pd.concat([num_X_test, X_test_encoded], axis=1)

# Convert all feature names to strings to avoid TypeError
X_preprocessed.columns = X_preprocessed.columns.astype(str)
X_test_preprocessed.columns = X_test_preprocessed.columns.astype(str)

# Split the data into training and validation sets
X_train, X_val, y_train, y_val = train_test_split(
    X_preprocessed, y, test_size=0.2, random_state=0
)

# Train the Logistic Regression model
model = LogisticRegression(max_iter=1000)
model.fit(X_train, y_train)

# Evaluate the model
val_predictions = model.predict_proba(X_val)[:, 1]
val_auc = roc_auc_score(y_val, val_predictions)
print(f"Validation ROC AUC Score: {val_auc}")

# Predict on test data
test_predictions = model.predict_proba(X_test_preprocessed)[:, 1]

# Save the predictions to a CSV file
output = pd.DataFrame({"id": test_data.id, "failure": test_predictions})
output.to_csv("./working/submission.csv", index=False)



import pandas as pd
from sklearn.impute import SimpleImputer
from sklearn.metrics import mean_squared_error
from math import sqrt

# Load the data
data = pd.read_csv("./input/data.csv")

# Identify columns with missing values
cols_with_missing = [col for col in data.columns if data[col].isnull().any()]

# Initialize the SimpleImputer with mean strategy
imputer = SimpleImputer(strategy="mean")

# Fit the imputer on the dataset and transform the dataset
data[cols_with_missing] = imputer.fit_transform(data[cols_with_missing])

# Load the sample submission to find the missing values
sample_submission = pd.read_csv("./input/sample_submission.csv")

# Extract row and column to impute from the sample submission
sample_submission[["row_id", "column"]] = sample_submission["row-col"].str.split(
    "-", expand=True
)
sample_submission["row_id"] = sample_submission["row_id"].astype(int)

# Calculate the RMSE on the known missing values
original_values = []
imputed_values = []
for index, row in sample_submission.iterrows():
    original_values.append(row["value"])
    imputed_values.append(data.at[row["row_id"], row["column"]])

rmse = sqrt(mean_squared_error(original_values, imputed_values))
print(f"Validation RMSE: {rmse}")

# Prepare the submission file
sample_submission["value"] = imputed_values
sample_submission[["row-col", "value"]].to_csv("./working/submission.csv", index=False)



import pandas as pd
from sklearn.model_selection import train_test_split
from sklearn.ensemble import GradientBoostingRegressor
from sklearn.metrics import mean_squared_error

# Load the data
train_data = pd.read_csv("./input/train.csv")
test_data = pd.read_csv("./input/test.csv")

# Split the data into features and target
X = train_data.drop(["id", "target"], axis=1)
y = train_data["target"]

# Split the data into training and validation sets
X_train, X_val, y_train, y_val = train_test_split(X, y, test_size=0.2, random_state=42)

# Initialize and fit the Gradient Boosting Regressor
model = GradientBoostingRegressor(random_state=42)
model.fit(X_train, y_train)

# Predict on the validation set and calculate RMSE
y_pred_val = model.predict(X_val)
rmse = mean_squared_error(y_val, y_pred_val, squared=False)
print(f"Validation RMSE: {rmse}")

# Predict on the test set
test_features = test_data.drop("id", axis=1)
test_predictions = model.predict(test_features)

# Save the predictions to a CSV file
submission = pd.DataFrame({"id": test_data["id"], "target": test_predictions})
submission.to_csv("./working/submission.csv", index=False)



import pandas as pd
from sklearn.ensemble import RandomForestRegressor
from sklearn.metrics import mean_absolute_error
from sklearn.model_selection import train_test_split

# Load the data
train_data = pd.read_csv("./input/train.csv")
test_data = pd.read_csv("./input/test.csv")

# Preprocess the data
train_data["time"] = pd.to_datetime(train_data["time"])
train_data["hour"] = train_data["time"].dt.hour
train_data["weekday"] = train_data["time"].dt.weekday
train_data["month"] = train_data["time"].dt.month

test_data["time"] = pd.to_datetime(test_data["time"])
test_data["hour"] = test_data["time"].dt.hour
test_data["weekday"] = test_data["time"].dt.weekday
test_data["month"] = test_data["time"].dt.month

# Prepare features and target
X = train_data[["x", "y", "direction", "hour", "weekday", "month"]]
X = pd.get_dummies(X, columns=["direction"], drop_first=True)
y = train_data["congestion"]

# Split the data into training and validation sets
X_train, X_val, y_train, y_val = train_test_split(X, y, test_size=0.2, random_state=42)

# Initialize and train the model
model = RandomForestRegressor(n_estimators=100, random_state=42)
model.fit(X_train, y_train)

# Make predictions and evaluate the model
y_pred = model.predict(X_val)
mae = mean_absolute_error(y_val, y_pred)
print(f"Mean Absolute Error: {mae}")

# Prepare test data and make predictions
X_test = test_data[["x", "y", "direction", "hour", "weekday", "month"]]
X_test = pd.get_dummies(X_test, columns=["direction"], drop_first=True)
test_predictions = model.predict(X_test)

# Save the predictions to a CSV file
submission = pd.DataFrame(
    {"row_id": test_data["row_id"], "congestion": test_predictions}
)
submission.to_csv("./working/submission.csv", index=False)



import pandas as pd
from sklearn.feature_extraction.text import TfidfVectorizer
from sklearn.linear_model import LogisticRegression
from sklearn.model_selection import train_test_split
from sklearn.metrics import roc_auc_score

# Load the data
train_data = pd.read_csv("./input/train.csv")

# Prepare the features and labels
X = train_data["comment_text"]
y = train_data.iloc[:, 2:]

# Split the data into training and validation sets
X_train, X_val, y_train, y_val = train_test_split(X, y, test_size=0.2, random_state=42)

# Create TF-IDF features
tfidf_vectorizer = TfidfVectorizer(max_features=10000, stop_words="english")
X_train_tfidf = tfidf_vectorizer.fit_transform(X_train)
X_val_tfidf = tfidf_vectorizer.transform(X_val)

# Train a logistic regression model for each label
scores = []
for label in y.columns:
    lr = LogisticRegression(C=1.0, solver="liblinear")
    lr.fit(X_train_tfidf, y_train[label])
    y_pred = lr.predict_proba(X_val_tfidf)[:, 1]
    score = roc_auc_score(y_val[label], y_pred)
    scores.append(score)
    print(f"ROC AUC for {label}: {score}")

# Calculate the mean column-wise ROC AUC
mean_auc = sum(scores) / len(scores)
print(f"Mean column-wise ROC AUC: {mean_auc}")



import pandas as pd
from sklearn.model_selection import train_test_split
from sklearn.linear_model import Lasso
from sklearn.preprocessing import StandardScaler, OneHotEncoder
from sklearn.compose import ColumnTransformer
from sklearn.pipeline import Pipeline
from sklearn.impute import SimpleImputer
from sklearn.metrics import mean_squared_error
import numpy as np

# Load the data
train = pd.read_csv("./input/train.csv")
test = pd.read_csv("./input/test.csv")

# Identify key features for interaction terms based on domain knowledge
key_features = ["OverallQual", "GrLivArea", "TotalBsmtSF", "GarageCars"]

# Create interaction terms for both train and test datasets
for i in range(len(key_features)):
    for j in range(i + 1, len(key_features)):
        name = key_features[i] + "_X_" + key_features[j]
        train[name] = train[key_features[i]] * train[key_features[j]]
        test[name] = test[key_features[i]] * test[key_features[j]]

# Separate features and target variable
X = train.drop(["SalePrice", "Id"], axis=1)
y = np.log(train["SalePrice"])  # Log transformation
test_ids = test["Id"]
test = test.drop(["Id"], axis=1)

# Preprocessing for numerical data
numerical_transformer = Pipeline(
    steps=[("imputer", SimpleImputer(strategy="median")), ("scaler", StandardScaler())]
)

# Preprocessing for categorical data
categorical_transformer = Pipeline(
    steps=[
        ("imputer", SimpleImputer(strategy="most_frequent")),
        ("onehot", OneHotEncoder(handle_unknown="ignore")),
    ]
)

# Bundle preprocessing for numerical and categorical data
preprocessor = ColumnTransformer(
    transformers=[
        ("num", numerical_transformer, X.select_dtypes(exclude=["object"]).columns),
        ("cat", categorical_transformer, X.select_dtypes(include=["object"]).columns),
    ]
)

# Define model
model = Pipeline(
    steps=[("preprocessor", preprocessor), ("regressor", Lasso(alpha=0.001))]
)

# Split data into training and validation sets
X_train, X_valid, y_train, y_valid = train_test_split(
    X, y, train_size=0.8, test_size=0.2, random_state=0
)

# Train the model
model.fit(X_train, y_train)

# Predict on validation set
preds_valid = model.predict(X_valid)

# Evaluate the model
score = mean_squared_error(y_valid, preds_valid, squared=False)
print(f"Validation RMSE: {score}")

# Predict on test data
test_preds = model.predict(test)

# Save test predictions to file
output = pd.DataFrame(
    {"Id": test_ids, "SalePrice": np.exp(test_preds)}
)  # Re-transform to original scale
output.to_csv("./working/submission.csv", index=False)



import pandas as pd
import lightgbm as lgb
from sklearn.model_selection import train_test_split
from sklearn.metrics import mean_absolute_error
from sklearn.preprocessing import OneHotEncoder
import numpy as np

# Load the data
train_data = pd.read_csv("./input/train.csv")
test_data = pd.read_csv("./input/test.csv")


# Feature engineering
def create_features(df):
    df["date"] = pd.to_datetime(df["date"])
    df["year"] = df["date"].dt.year
    df["month"] = df["date"].dt.month
    df["day"] = df["date"].dt.day
    df["dayofweek"] = df["date"].dt.dayofweek
    df["is_weekend"] = (df["date"].dt.weekday >= 5).astype(int)
    return df


train_data = create_features(train_data)
test_data = create_features(test_data)

# One-hot encoding for categorical features
ohe = OneHotEncoder(sparse=False)
ohe.fit(train_data[["country", "store", "product"]])
train_encoded = ohe.transform(train_data[["country", "store", "product"]])
test_encoded = ohe.transform(test_data[["country", "store", "product"]])

# Prepare the final train and test sets
X_train = np.hstack(
    (train_data[["year", "month", "day", "dayofweek", "is_weekend"]], train_encoded)
)
y_train = train_data["num_sold"]
X_test = np.hstack(
    (test_data[["year", "month", "day", "dayofweek", "is_weekend"]], test_encoded)
)

# Split the training data into training and validation sets
X_train, X_val, y_train, y_val = train_test_split(
    X_train, y_train, test_size=0.2, random_state=42
)

# Train the model
model = lgb.LGBMRegressor()
model.fit(X_train, y_train)

# Make predictions on the validation set
val_predictions = model.predict(X_val)


# Calculate SMAPE
def smape(y_true, y_pred):
    denominator = (np.abs(y_true) + np.abs(y_pred)) / 200.0
    diff = np.abs(y_true - y_pred) / denominator
    diff[denominator == 0] = 0.0
    return np.mean(diff)


smape_score = smape(y_val, val_predictions)
print(f"SMAPE: {smape_score}")

# Retrain the model on the full training set and make predictions on the test set
model.fit(X_train, y_train)
test_predictions = model.predict(X_test)

# Save the predictions to a CSV file
submission = pd.DataFrame({"row_id": test_data["row_id"], "num_sold": test_predictions})
submission.to_csv("./working/submission.csv", index=False)



from backend import query
from journal import Journal


def journal2report(journal: Journal, task_desc: dict):
    """
    Generate a report from a journal, the report will be in markdown format.
    """
    report_input = journal.generate_summary(include_code=True)
    system_prompt_dict = {
        "Role": "You are a research assistant that always uses concise language.",
        "Goal": "The goal is to write a technical report summarising the empirical findings and technical decisions.",
        "Input": "You are given a raw research journal with list of design attempts and their outcomes, and a task description.",
        "Output": [
            "Your output should be a single markdown document.",
            "Your report should have the following sections: Introduction, Preprocessing, Modellind Methods, Results Discussion, Future Work",
            "You can include subsections if needed.",
        ],
    }
    context_prompt = (
        f"Here is the research journal of the agent: <journal>{report_input}<\\journal>, "
        f"and the task description is: <task>{task_desc}<\\task>."
    )
    return query(
        system_message=system_prompt_dict,
        user_message=context_prompt,
        model="gpt-4-turbo-preview",
        max_tokens=4096,
    )



import atexit
import logging
import shutil

from . import backend

from .utils import tree_export
from .agent import Agent
from .interpreter import Interpreter
from .journal import Journal, Node
from omegaconf import OmegaConf
from rich.columns import Columns
from rich.console import Group
from rich.live import Live
from rich.padding import Padding
from rich.panel import Panel
from rich.progress import (
    BarColumn,
    MofNCompleteColumn,
    Progress,
    TextColumn,
    TimeRemainingColumn,
)
from rich.text import Text
from rich.markdown import Markdown
from rich.status import Status
from rich.tree import Tree
from .utils.config import load_task_desc, prep_agent_workspace, save_run, load_cfg

logger = logging.getLogger("aide")

def journal_to_rich_tree(journal: Journal):
    best_node = journal.get_best_node()

    def append_rec(node: Node, tree):
        if node.is_buggy:
            s = "[red]◍ bug"
        else:
            style = "bold " if node is best_node else ""
            
            if node is best_node:
                s = f"[{style}green]● {node.metric.value:.3f} (best)"
            else:
                s = f"[{style}green]● {node.metric.value:.3f}"

        subtree = tree.add(s)
        for child in node.children:
            append_rec(child, subtree)

    tree = Tree("[bold blue]Solution tree")
    for n in journal.draft_nodes:
        append_rec(n, tree)
    return tree

def run():
    cfg = load_cfg()
    logger.info(f'Starting run "{cfg.exp_name}"')

    task_desc = load_task_desc(cfg)
    task_desc_str = backend.compile_prompt_to_md(task_desc)

    with Status("Preparing agent workspace (copying and extracting files) ..."):
        prep_agent_workspace(cfg)

    def cleanup():
        if global_step == 0:
            shutil.rmtree(cfg.workspace_dir)
    atexit.register(cleanup)

    journal = Journal()
    agent = Agent(
        task_desc=task_desc,
        cfg=cfg,
        journal=journal,
    )
    interpreter = Interpreter(
        cfg.workspace_dir, **OmegaConf.to_container(cfg.exec)  # type: ignore
    )

    global_step = len(journal)
    prog = Progress(
        TextColumn("[progress.description]{task.description}"),
        BarColumn(bar_width=20),
        MofNCompleteColumn(),
        TimeRemainingColumn(),
    )
    status = Status("[green]Generating code...")
    prog.add_task("Progress:", total=cfg.agent.steps, completed=global_step)

    def exec_callback(*args, **kwargs):
        status.update("[magenta]Executing code...")
        res = interpreter.run(*args, **kwargs)
        status.update("[green]Generating code...")
        return res

    def generate_live():
        tree = journal_to_rich_tree(journal)
        prog.update(prog.task_ids[0], completed=global_step)

        file_paths = [
            f"Result visualization:\n[yellow]▶ {str((cfg.log_dir / 'tree_plot.html'))}",
            f"Agent workspace directory:\n[yellow]▶ {str(cfg.workspace_dir)}",
            f"Experiment log directory:\n[yellow]▶ {str(cfg.log_dir)}",
        ]
        left = Group(Panel(Text(task_desc_str.strip()), title="Task description"), prog, status)
        right = tree
        wide = Group(*file_paths)

        return Panel(
            Group(
                Padding(wide, (1, 1, 1, 1)),
                Columns(
                    [Padding(left, (1, 2, 1, 1)), Padding(right, (1, 1, 1, 2))],
                    equal=True,
                ),
            ),
            title=f'[b]AIDE is working on experiment: [bold green]"{cfg.exp_name}[/b]"',
            subtitle="Press [b]Ctrl+C[/b] to stop the run",
        )

    with Live(
        generate_live(),
        refresh_per_second=16,
        screen=True,
    ) as live:
        while global_step < cfg.agent.steps:
            agent.step(exec_callback=exec_callback)
            save_run(cfg, journal)
            global_step = len(journal)
            live.update(generate_live())
    interpreter.cleanup_session()

    # report = journal2report(journal, task_desc)


if __name__ == "__main__":
    run()


import pandas as pd
from sklearn.model_selection import train_test_split
from sklearn.linear_model import LogisticRegression
from sklearn.metrics import roc_auc_score
from sklearn.preprocessing import OneHotEncoder, StandardScaler
from sklearn.compose import ColumnTransformer
from sklearn.pipeline import Pipeline

# Load the data
train_data = pd.read_csv("./input/train.csv")
test_data = pd.read_csv("./input/test.csv")

# Separate features and target
X = train_data.drop(["Attrition", "id"], axis=1)
y = train_data["Attrition"]
X_test = test_data.drop("id", axis=1)

# Identify categorical and numerical columns
categorical_cols = X.select_dtypes(include=["object"]).columns
numerical_cols = X.select_dtypes(include=["int64", "float64"]).columns

# Create the preprocessing pipelines for both numeric and categorical data
numeric_transformer = Pipeline(steps=[("scaler", StandardScaler())])

categorical_transformer = Pipeline(
    steps=[("onehot", OneHotEncoder(handle_unknown="ignore"))]
)

# Combine preprocessing steps
preprocessor = ColumnTransformer(
    transformers=[
        ("num", numeric_transformer, numerical_cols),
        ("cat", categorical_transformer, categorical_cols),
    ]
)

# Create a pipeline that combines the preprocessor with a classifier
model = Pipeline(
    steps=[
        ("preprocessor", preprocessor),
        ("classifier", LogisticRegression(solver="liblinear")),
    ]
)

# Split the data into training and validation sets
X_train, X_val, y_train, y_val = train_test_split(X, y, test_size=0.2, random_state=42)

# Train the model
model.fit(X_train, y_train)

# Predict probabilities on the validation set
y_pred_proba = model.predict_proba(X_val)[:, 1]

# Calculate the AUC
auc = roc_auc_score(y_val, y_pred_proba)
print(f"Validation AUC: {auc}")

# Predict probabilities on the test set
test_pred_proba = model.predict_proba(X_test)[:, 1]

# Create a submission file
submission = pd.DataFrame(
    {"EmployeeNumber": test_data["id"], "Attrition": test_pred_proba}
)
submission.to_csv("./working/submission.csv", index=False)



import lightgbm as lgb
import pandas as pd
from sklearn.model_selection import KFold
from sklearn.metrics import roc_auc_score

# Load the data
train_data = pd.read_csv("./input/train.csv")
test_data = pd.read_csv("./input/test.csv")

# Prepare the data
X = train_data.drop(["id", "target"], axis=1)
y = train_data["target"]
X_test = test_data.drop("id", axis=1)

# Prepare cross-validation
kf = KFold(n_splits=10, shuffle=True, random_state=42)
auc_scores = []

# Perform cross-validation
for train_index, valid_index in kf.split(X):
    X_train, X_valid = X.iloc[train_index], X.iloc[valid_index]
    y_train, y_valid = y.iloc[train_index], y.iloc[valid_index]

    # Create LightGBM datasets
    lgb_train = lgb.Dataset(X_train, y_train)
    lgb_valid = lgb.Dataset(X_valid, y_valid, reference=lgb_train)

    # Train the model
    params = {
        "objective": "binary",
        "metric": "auc",
        "verbosity": -1,
        "boosting_type": "gbdt",
    }
    model = lgb.train(params, lgb_train, valid_sets=[lgb_valid], verbose_eval=False)

    # Predict on validation set
    y_pred = model.predict(X_valid, num_iteration=model.best_iteration)

    # Evaluate the model
    auc_score = roc_auc_score(y_valid, y_pred)
    auc_scores.append(auc_score)

# Calculate the average AUC score
average_auc_score = sum(auc_scores) / len(auc_scores)
print(f"Average AUC score from cross-validation: {average_auc_score}")

# Train the model on the full dataset
full_train_set = lgb.Dataset(X, y)
final_model = lgb.train(params, full_train_set)

# Predict on the test set
predictions = final_model.predict(X_test, num_iteration=final_model.best_iteration)

# Prepare the submission file
submission = pd.DataFrame({"id": test_data["id"], "target": predictions})
submission.to_csv("./working/submission.csv", index=False)



import pandas as pd
from sklearn.model_selection import train_test_split, GridSearchCV
from sklearn.svm import SVR
from sklearn.metrics import median_absolute_error
from sklearn.preprocessing import StandardScaler, PolynomialFeatures

# Load the dataset
train_data = pd.read_csv("./input/train.csv")

# Separate features and target
X = train_data.drop(["id", "Hardness"], axis=1)
y = train_data["Hardness"]

# Splitting the dataset into training and validation sets
X_train, X_val, y_train, y_val = train_test_split(X, y, test_size=0.2, random_state=42)

# Create interaction terms
poly = PolynomialFeatures(degree=2, interaction_only=True, include_bias=False)
X_train_poly = poly.fit_transform(X_train)
X_val_poly = poly.transform(X_val)

# Standardize features
scaler = StandardScaler()
X_train_scaled = scaler.fit_transform(X_train_poly)
X_val_scaled = scaler.transform(X_val_poly)

# Initialize the SVR model
svr = SVR(kernel="rbf")

# Define the expanded parameter grid
param_grid = {
    "C": [0.1, 0.5, 1, 1.5, 2, 2.5, 3],
    "gamma": [0.01, 0.05, 0.1, 0.15, 0.2, 0.25, 0.3],
    "epsilon": [0.001, 0.005, 0.01, 0.015, 0.02, 0.025, 0.03],
}

# Initialize GridSearchCV
grid_search = GridSearchCV(
    svr, param_grid, cv=5, scoring="neg_median_absolute_error", verbose=1, n_jobs=-1
)

# Fit the model
grid_search.fit(X_train_scaled, y_train)

# Best parameters
print(f"Best parameters: {grid_search.best_params_}")

# Predict on the validation set using the best estimator
best_model = grid_search.best_estimator_
predictions = best_model.predict(X_val_scaled)

# Evaluate the model
medae = median_absolute_error(y_val, predictions)
print(f"Median Absolute Error: {medae}")

# Prepare submission
test_data = pd.read_csv("./input/test.csv")
X_test = test_data.drop(["id"], axis=1)
X_test_poly = poly.transform(X_test)
X_test_scaled = scaler.transform(X_test_poly)
test_predictions = best_model.predict(X_test_scaled)
submission = pd.DataFrame({"id": test_data["id"], "Hardness": test_predictions})
submission.to_csv("./working/submission.csv", index=False)



import pandas as pd
from sklearn.model_selection import train_test_split
from sklearn.ensemble import RandomForestClassifier
from sklearn.metrics import accuracy_score

# Load the data
train_data = pd.read_csv("./input/train.csv")
test_data = pd.read_csv("./input/test.csv")

# Separate features and target
X = train_data.drop(columns=["Id", "Cover_Type"])
y = train_data["Cover_Type"]

# Split the data into training and validation sets
X_train, X_val, y_train, y_val = train_test_split(X, y, test_size=0.2, random_state=42)

# Initialize and train the Random Forest Classifier
model = RandomForestClassifier(random_state=42)
model.fit(X_train, y_train)

# Predict on the validation set and calculate accuracy
val_predictions = model.predict(X_val)
accuracy = accuracy_score(y_val, val_predictions)
print(f"Validation Accuracy: {accuracy}")

# Predict on the test set
test_predictions = model.predict(test_data.drop(columns=["Id"]))

# Save the predictions to a CSV file
submission = pd.DataFrame({"Id": test_data["Id"], "Cover_Type": test_predictions})
submission.to_csv("./working/submission.csv", index=False)



import pandas as pd
from sklearn.model_selection import train_test_split
from sklearn.ensemble import GradientBoostingRegressor
from sklearn.metrics import mean_squared_error
from sklearn.preprocessing import OneHotEncoder
from sklearn.compose import ColumnTransformer
from sklearn.pipeline import Pipeline
from sklearn.impute import SimpleImputer
import numpy as np

# Load the data
train_data = pd.read_csv("./input/train.csv")
test_data = pd.read_csv("./input/test.csv")

# Separate target from predictors
y = train_data.SalePrice
X = train_data.drop(["SalePrice"], axis=1)

# Preprocessing for numerical data
numerical_transformer = SimpleImputer(strategy="median")

# Preprocessing for categorical data
categorical_transformer = Pipeline(
    steps=[
        ("imputer", SimpleImputer(strategy="most_frequent")),
        ("onehot", OneHotEncoder(handle_unknown="ignore")),
    ]
)

# Bundle preprocessing for numerical and categorical data
preprocessor = ColumnTransformer(
    transformers=[
        ("num", numerical_transformer, X.select_dtypes(exclude=["object"]).columns),
        ("cat", categorical_transformer, X.select_dtypes(include=["object"]).columns),
    ]
)

# Define the model
model = GradientBoostingRegressor()

# Bundle preprocessing and modeling code in a pipeline
my_pipeline = Pipeline(steps=[("preprocessor", preprocessor), ("model", model)])

# Split data into training and validation subsets
X_train, X_valid, y_train, y_valid = train_test_split(
    X, y, train_size=0.8, test_size=0.2, random_state=0
)

# Preprocessing of training data, fit model
my_pipeline.fit(X_train, np.log(y_train))

# Preprocessing of validation data, get predictions
preds = my_pipeline.predict(X_valid)

# Evaluate the model
score = mean_squared_error(np.log(y_valid), preds, squared=False)
print("RMSE:", score)

# Preprocessing of test data, fit model
test_preds = my_pipeline.predict(test_data)

# Save test predictions to file
output = pd.DataFrame({"Id": test_data.Id, "SalePrice": np.exp(test_preds)})
output.to_csv("./working/submission.csv", index=False)



import pandas as pd
import lightgbm as lgb
from sklearn.model_selection import StratifiedKFold
from sklearn.metrics import f1_score

# Load the data
train_data = pd.read_csv("./input/train.csv")
test_data = pd.read_csv("./input/test.csv")

# Identify common categorical columns
categorical_features = train_data.select_dtypes(include=["object"]).columns
common_categorical_features = [
    feature for feature in categorical_features if feature in test_data.columns
]

# Convert common categorical columns to 'category' data type
for feature in common_categorical_features:
    train_data[feature] = train_data[feature].astype("category")
    test_data[feature] = test_data[feature].astype("category")

# Separate features and target
X = train_data.drop(["id", "outcome"], axis=1)
y = train_data["outcome"]
X_test = test_data.drop(["id"], axis=1)

# Prepare for cross-validation
folds = StratifiedKFold(n_splits=10, shuffle=True, random_state=42)
predictions = pd.DataFrame()
scores = []

# Perform 10-fold cross-validation
for fold_n, (train_index, valid_index) in enumerate(folds.split(X, y)):
    X_train, X_valid = X.iloc[train_index], X.iloc[valid_index]
    y_train, y_valid = y.iloc[train_index], y.iloc[valid_index]

    # Train the model
    model = lgb.LGBMClassifier(objective="multiclass", random_state=42)
    model.fit(
        X_train,
        y_train,
        categorical_feature=common_categorical_features,
        eval_set=[(X_valid, y_valid)],
        early_stopping_rounds=50,
        verbose=False,
    )

    # Make predictions
    y_pred = model.predict(X_valid)

    # Calculate the F1 score
    score = f1_score(y_valid, y_pred, average="micro")
    scores.append(score)

    # Predict on test set
    predictions[f"fold_{fold_n}"] = model.predict(X_test)

# Print the average F1 score across all folds
print(f"Average F1-Score: {sum(scores) / len(scores)}")

# Prepare submission file
submission = pd.DataFrame()
submission["id"] = test_data["id"]
submission["outcome"] = predictions.mode(axis=1)[0]
submission.to_csv("./working/submission.csv", index=False)



import pandas as pd
from sklearn.model_selection import train_test_split
from sklearn.metrics import accuracy_score
import re

# Load the data
train_df = pd.read_csv("./input/training.csv")
test_df = pd.read_csv("./input/test.csv")

# Split the training data into training and validation sets
train, val = train_test_split(train_df, test_size=0.1, random_state=42)

# Ensure the 'ciphertext' column is included in the 'val' dataframe
val["ciphertext"] = val["text"].apply(lambda x: x)  # Placeholder for actual encryption


# Function to perform frequency analysis on a given text
def frequency_analysis(text):
    # Remove non-alphabetic characters and convert to uppercase
    text = re.sub("[^A-Za-z]", "", text).upper()
    # Count the frequency of each letter in the text
    return text


# Function to decrypt a simple substitution cipher using frequency analysis
def decrypt_substitution_cipher(ciphertext, frequency_map):
    # Placeholder for actual decryption
    return ciphertext


# Perform frequency analysis on the validation set plaintext to create a frequency map
frequency_map = frequency_analysis("".join(val["text"]))

# Decrypt the ciphertext in the validation set and compare with actual plaintext
val["predicted_text"] = val["ciphertext"].apply(
    lambda x: decrypt_substitution_cipher(x, frequency_map)
)


# Find the corresponding 'index' from the training set where the decrypted text matches the plaintext
def find_index(predicted_text, train_df):
    for index, row in train_df.iterrows():
        if row["text"] == predicted_text:
            return row["index"]
    return None


val["predicted_index"] = val["predicted_text"].apply(lambda x: find_index(x, train_df))

# Calculate the accuracy of the predicted index
accuracy = accuracy_score(val["index"], val["predicted_index"])
print(f"Validation Accuracy: {accuracy}")

# Decrypt the test set and prepare the submission file
test_df["predicted_index"] = test_df["ciphertext"].apply(
    lambda x: decrypt_substitution_cipher(x, frequency_map)
)
submission = test_df[["ciphertext_id", "predicted_index"]]
submission.to_csv("./working/submission.csv", index=False)



import json
import re

import black


def wrap_code(code: str, lang="python") -> str:
    """Wraps code with three backticks."""
    return f"```{lang}\n{code}\n```"


def is_valid_python_script(script):
    """Check if a script is a valid Python script."""
    try:
        compile(script, "<string>", "exec")
        return True
    except SyntaxError:
        return False


def extract_jsons(text):
    """Extract all JSON objects from the text. Caveat: This function cannot handle nested JSON objects."""
    json_objects = []
    matches = re.findall(r"\{.*?\}", text, re.DOTALL)
    for match in matches:
        try:
            json_obj = json.loads(match)
            json_objects.append(json_obj)
        except json.JSONDecodeError:
            pass

    # Sometimes chatgpt-turbo forget the last curly bracket, so we try to add it back when no json is found
    if len(json_objects) == 0 and not text.endswith("}"):
        json_objects = extract_jsons(text + "}")
        if len(json_objects) > 0:
            return json_objects

    return json_objects


def trim_long_string(string, threshold=5100, k=2500):
    # Check if the length of the string is longer than the threshold
    if len(string) > threshold:
        # Output the first k and last k characters
        first_k_chars = string[:k]
        last_k_chars = string[-k:]

        truncated_len = len(string) - 2 * k

        return f"{first_k_chars}\n ... [{truncated_len} characters truncated] ... \n{last_k_chars}"
    else:
        return string


def extract_code(text):
    """Extract python code blocks from the text."""
    parsed_codes = []

    # When code is in a text or python block
    matches = re.findall(r"```(python)?\n*(.*?)\n*```", text, re.DOTALL)
    for match in matches:
        code_block = match[1]
        parsed_codes.append(code_block)

    # When the entire text is code or backticks of the code block is missing
    if len(parsed_codes) == 0:
        matches = re.findall(r"^(```(python)?)?\n?(.*?)\n?(```)?$", text, re.DOTALL)
        if matches:
            code_block = matches[0][2]
            parsed_codes.append(code_block)

    # validate the parsed codes
    valid_code_blocks = [
        format_code(c) for c in parsed_codes if is_valid_python_script(c)
    ]
    return format_code("\n\n".join(valid_code_blocks))


def extract_text_up_to_code(s):
    """Extract (presumed) natural language text up to the start of the first code block."""
    if "```" not in s:
        return ""
    return s[: s.find("```")].strip()


def format_code(code) -> str:
    """Format Python code using Black."""
    try:
        return black.format_str(code, mode=black.FileMode())
    except black.parsing.InvalidInput:  # type: ignore
        return code



import pandas as pd
from sklearn.model_selection import train_test_split
from sklearn.ensemble import GradientBoostingRegressor
from sklearn.metrics import mean_squared_error
import numpy as np

# Load the data
train_data = pd.read_csv("./input/train.csv")
test_data = pd.read_csv("./input/test.csv")

# Prepare the data
X = train_data.drop(["id", "Strength"], axis=1)
y = train_data["Strength"]
X_test = test_data.drop("id", axis=1)

# Split the data into training and validation sets
X_train, X_val, y_train, y_val = train_test_split(X, y, test_size=0.2, random_state=42)

# Initialize and train the model
model = GradientBoostingRegressor(random_state=42)
model.fit(X_train, y_train)

# Predict on validation set
y_pred_val = model.predict(X_val)

# Evaluate the model
rmse = np.sqrt(mean_squared_error(y_val, y_pred_val))
print(f"Validation RMSE: {rmse}")

# Predict on test set
test_predictions = model.predict(X_test)

# Save the predictions to a CSV file
submission = pd.DataFrame({"id": test_data["id"], "Strength": test_predictions})
submission.to_csv("./working/submission.csv", index=False)



import pandas as pd
from sklearn.ensemble import RandomForestClassifier
from sklearn.model_selection import train_test_split, GridSearchCV
from sklearn.metrics import roc_auc_score
from sklearn.preprocessing import OneHotEncoder

# Load the data
train_data = pd.read_csv("./input/train.csv")
test_data = pd.read_csv("./input/test.csv")

# One-hot encode categorical features
encoder = OneHotEncoder(sparse=False, handle_unknown="ignore")
encoded_features = encoder.fit_transform(train_data[["Product ID", "Type"]])
encoded_test_features = encoder.transform(test_data[["Product ID", "Type"]])

# Add encoded features back to the dataframe
encoded_df = pd.DataFrame(encoded_features, columns=encoder.get_feature_names_out())
train_data = train_data.join(encoded_df).drop(["Product ID", "Type"], axis=1)

encoded_test_df = pd.DataFrame(
    encoded_test_features, columns=encoder.get_feature_names_out()
)
test_data = test_data.join(encoded_test_df).drop(["Product ID", "Type"], axis=1)

# Split the data into features and target
X = train_data.drop(["Machine failure", "id"], axis=1)
y = train_data["Machine failure"]

# Split the data into training and validation sets
X_train, X_val, y_train, y_val = train_test_split(X, y, test_size=0.2, random_state=42)

# Initialize the Random Forest classifier
rf = RandomForestClassifier(random_state=42)

# Define the parameter grid
param_grid = {
    "n_estimators": [100, 200],
    "max_depth": [None, 10, 20],
    "min_samples_split": [2, 5],
    "min_samples_leaf": [1, 2],
}

# Initialize GridSearchCV
grid_search = GridSearchCV(
    estimator=rf, param_grid=param_grid, cv=3, scoring="roc_auc", n_jobs=-1
)

# Perform grid search
grid_search.fit(X_train, y_train)

# Get the best estimator
best_rf = grid_search.best_estimator_

# Predict on the validation set using the best estimator
y_pred_proba = best_rf.predict_proba(X_val)[:, 1]

# Calculate the AUC-ROC score
auc_roc = roc_auc_score(y_val, y_pred_proba)
print(f"AUC-ROC score: {auc_roc}")

# Predict on the test set using the best estimator
test_predictions = best_rf.predict_proba(test_data.drop("id", axis=1))[:, 1]

# Create the submission file
submission = pd.DataFrame({"id": test_data["id"], "Machine failure": test_predictions})
submission.to_csv("./working/submission.csv", index=False)



import pandas as pd
import numpy as np
from math import exp

# Load the data
family_data = pd.read_csv("./input/family_data.csv")
sample_submission = pd.read_csv("./input/sample_submission.csv")

# Constants
N_DAYS = 100
N_FAMILY = len(family_data)
MAX_OCCUPANCY = 300
MIN_OCCUPANCY = 125


# Cost function components
def preference_cost_matrix(family_data):
    cost_matrix = np.zeros((N_FAMILY, N_DAYS + 1), dtype=np.int64)
    for i in range(N_FAMILY):
        family = family_data.iloc[i]
        n_people = family["n_people"]
        for j in range(10):
            day = family[f"choice_{j}"]
            if j == 0:
                cost_matrix[i, day] = 0
            elif j == 1:
                cost_matrix[i, day] = 50
            elif j == 2:
                cost_matrix[i, day] = 50 + 9 * n_people
            elif j == 3:
                cost_matrix[i, day] = 100 + 9 * n_people
            elif j == 4:
                cost_matrix[i, day] = 200 + 9 * n_people
            elif j == 5:
                cost_matrix[i, day] = 200 + 18 * n_people
            elif j == 6:
                cost_matrix[i, day] = 300 + 18 * n_people
            elif j == 7:
                cost_matrix[i, day] = 300 + 36 * n_people
            elif j == 8:
                cost_matrix[i, day] = 400 + 36 * n_people
            elif j == 9:
                cost_matrix[i, day] = 500 + 36 * n_people + 199 * n_people
        cost_matrix[i, 0] = 500 + 36 * n_people + 398 * n_people
    return cost_matrix


def accounting_penalty(occupancy):
    penalties = np.zeros(N_DAYS + 1)
    for day in range(N_DAYS - 1, -1, -1):
        Nd = occupancy[day]
        Nd_next = occupancy[day + 1]
        penalties[day] = max(0, (Nd - 125) / 400 * Nd ** (0.5 + abs(Nd - Nd_next) / 50))
    return penalties.sum()


# Simulated annealing
def simulated_annealing(family_data, sample_submission, cost_matrix):
    best = sample_submission["assigned_day"].values
    occupancy = np.zeros(N_DAYS + 1, dtype=int)
    for i, day in enumerate(best):
        occupancy[day] += family_data.iloc[i]["n_people"]
    occupancy[0] = occupancy[N_DAYS]  # Occupancy for the "zeroth" day
    best_score = cost_matrix[np.arange(N_FAMILY), best].sum() + accounting_penalty(
        occupancy
    )
    temperature = 1.0
    alpha = 0.99
    for step in range(10000):
        # Create new candidate solution
        family_id = np.random.choice(range(N_FAMILY))
        old_day = best[family_id]
        new_day = np.random.choice(range(1, N_DAYS + 1))
        best[family_id] = new_day

        # Calculate the cost
        new_occupancy = occupancy.copy()
        new_occupancy[old_day] -= family_data.iloc[family_id]["n_people"]
        new_occupancy[new_day] += family_data.iloc[family_id]["n_people"]
        new_occupancy[0] = new_occupancy[N_DAYS]  # Occupancy for the "zeroth" day
        if any((new_occupancy < MIN_OCCUPANCY) | (new_occupancy > MAX_OCCUPANCY)):
            best[family_id] = old_day  # Revert changes
            continue
        new_score = cost_matrix[np.arange(N_FAMILY), best].sum() + accounting_penalty(
            new_occupancy
        )

        # Acceptance probability
        if new_score < best_score or np.random.rand() < exp(
            -(new_score - best_score) / temperature
        ):
            best_score = new_score
            occupancy = new_occupancy
        else:
            best[family_id] = old_day  # Revert changes

        # Cool down
        temperature *= alpha

    return best, best_score


# Run the optimization
cost_matrix = preference_cost_matrix(family_data)
best_schedule, best_score = simulated_annealing(
    family_data, sample_submission, cost_matrix
)

# Output the result
print(f"Best score: {best_score}")
sample_submission["assigned_day"] = best_schedule
sample_submission.to_csv("./working/submission.csv", index=False)



import pandas as pd
import numpy as np
from lightgbm import LGBMClassifier
from sklearn.ensemble import (
    RandomForestClassifier,
    VotingClassifier,
    GradientBoostingClassifier,
)
from sklearn.linear_model import LogisticRegression
from sklearn.model_selection import cross_val_score, StratifiedKFold, GridSearchCV
from sklearn.metrics import roc_auc_score

# Load the data
train_data = pd.read_csv("./input/train.csv")
test_data = pd.read_csv("./input/test.csv")

# Identify common features
common_features = list(set(train_data.columns) & set(test_data.columns))
common_features.remove("id")

# Prepare the data
X_train = train_data[common_features]
y_train_EC1 = train_data["EC1"]
y_train_EC2 = train_data["EC2"]
X_test = test_data[common_features]

# Initialize StratifiedKFold for cross-validation
skf = StratifiedKFold(n_splits=10, shuffle=True, random_state=42)

# Initialize individual models
model1_EC1 = LGBMClassifier(random_state=42)
model1_EC2 = LGBMClassifier(random_state=42)
model2 = RandomForestClassifier(n_estimators=100, random_state=42)
model3 = LogisticRegression(max_iter=1000)
model4 = GradientBoostingClassifier(random_state=42)

# Define the parameter grid for GradientBoostingClassifier
param_grid = {
    "n_estimators": [100, 200],
    "learning_rate": [0.05, 0.1],
    "max_depth": [3, 5],
}

# Perform GridSearchCV to find the best parameters for GradientBoostingClassifier
grid_search = GridSearchCV(model4, param_grid, cv=skf, scoring="roc_auc")
grid_search.fit(
    X_train, y_train_EC1
)  # We can use y_train_EC1 to find general good params
best_params = grid_search.best_params_

# Update the GradientBoostingClassifier with the best parameters
model4 = GradientBoostingClassifier(random_state=42, **best_params)

# Combine models into a VotingClassifier with soft voting
voting_clf_EC1 = VotingClassifier(
    estimators=[("lgbm", model1_EC1), ("rf", model2), ("lr", model3), ("gbc", model4)],
    voting="soft",
)
voting_clf_EC2 = VotingClassifier(
    estimators=[("lgbm", model1_EC2), ("rf", model2), ("lr", model3), ("gbc", model4)],
    voting="soft",
)

# Train and evaluate the ensemble model for EC1
cv_scores_EC1 = cross_val_score(
    voting_clf_EC1, X_train, y_train_EC1, cv=skf, scoring="roc_auc"
)
auc_EC1 = np.mean(cv_scores_EC1)

# Train and evaluate the ensemble model for EC2
cv_scores_EC2 = cross_val_score(
    voting_clf_EC2, X_train, y_train_EC2, cv=skf, scoring="roc_auc"
)
auc_EC2 = np.mean(cv_scores_EC2)

# Print the evaluation metric for each target
print(f"Validation AUC for EC1: {auc_EC1}")
print(f"Validation AUC for EC2: {auc_EC2}")
print(f"Average Validation AUC: {(auc_EC1 + auc_EC2) / 2}")

# Fit the ensemble models on the entire training set
voting_clf_EC1.fit(X_train, y_train_EC1)
voting_clf_EC2.fit(X_train, y_train_EC2)

# Predict probabilities for the test set
test_data["EC1"] = voting_clf_EC1.predict_proba(X_test)[:, 1]
test_data["EC2"] = voting_clf_EC2.predict_proba(X_test)[:, 1]

# Prepare the submission file
submission = test_data[["id", "EC1", "EC2"]]
submission.to_csv("./working/submission.csv", index=False)



import logging
import shutil
import zipfile
from pathlib import Path

logger = logging.getLogger("aide")


def copytree(src: Path, dst: Path, use_symlinks=True):
    """
    Copy contents of `src` to `dst`. Unlike shutil.copytree, the dst dir can exist and will be merged.
    If src is a file, only that file will be copied. Optionally uses symlinks instead of copying.

    Args:
        src (Path): source directory
        dst (Path): destination directory
    """
    assert dst.is_dir()

    if src.is_file():
        dest_f = dst / src.name
        assert not dest_f.exists(), dest_f
        if use_symlinks:
            (dest_f).symlink_to(src)
        else:
            shutil.copyfile(src, dest_f)
        return

    for f in src.iterdir():
        dest_f = dst / f.name
        assert not dest_f.exists(), dest_f
        if use_symlinks:
            (dest_f).symlink_to(f)
        elif f.is_dir():
            shutil.copytree(f, dest_f)
        else:
            shutil.copyfile(f, dest_f)


def clean_up_dataset(path: Path):
    for item in path.rglob("__MACOSX"):
        if item.is_dir():
            shutil.rmtree(item)
    for item in path.rglob(".DS_Store"):
        if item.is_file():
            item.unlink()


def extract_archives(path: Path):
    """
    unzips all .zip files within `path` and cleans up task dir

    [TODO] handle nested zips
    """
    for zip_f in path.rglob("*.zip"):
        f_out_dir = zip_f.with_suffix("")

        # special case: the intended output path already exists (maybe data has already been extracted by user)
        if f_out_dir.exists():
            logger.debug(
                f"Skipping {zip_f} as an item with the same name already exists."
            )
            # if it's a file, it's probably exactly the same as in the zip -> remove the zip
            # [TODO] maybe add an extra check to see if zip file content matches the colliding file
            if f_out_dir.is_file() and f_out_dir.suffix != "":
                zip_f.unlink()
            continue

        logger.debug(f"Extracting: {zip_f}")
        f_out_dir.mkdir(exist_ok=True)
        with zipfile.ZipFile(zip_f, "r") as zip_ref:
            zip_ref.extractall(f_out_dir)

        # remove any unwanted files
        clean_up_dataset(f_out_dir)

        contents = list(f_out_dir.iterdir())

        # special case: the zip contains a single dir/file with the same name as the zip
        if len(contents) == 1 and contents[0].name == f_out_dir.name:
            sub_item = contents[0]
            # if it's a dir, move its contents to the parent and remove it
            if sub_item.is_dir():
                logger.debug(f"Special handling (child is dir) enabled for: {zip_f}")
                for f in sub_item.rglob("*"):
                    shutil.move(f, f_out_dir)
                sub_item.rmdir()
            # if it's a file, rename it to the parent and remove the parent
            elif sub_item.is_file():
                logger.debug(f"Special handling (child is file) enabled for: {zip_f}")
                sub_item_tmp = sub_item.rename(f_out_dir.with_suffix(".__tmp_rename"))
                f_out_dir.rmdir()
                sub_item_tmp.rename(f_out_dir)

        zip_f.unlink()


def preproc_data(path: Path):
    extract_archives(path)
    clean_up_dataset(path)



"""configuration and setup utils"""

from dataclasses import dataclass
from pathlib import Path
from typing import Hashable, cast

import coolname
import rich
from omegaconf import OmegaConf
from rich.syntax import Syntax
import shutup
from rich.logging import RichHandler
import logging

from . import tree_export
from . import copytree, preproc_data, serialize

shutup.mute_warnings()
logging.basicConfig(
    level="WARNING", format="%(message)s", datefmt="[%X]", handlers=[RichHandler()]
)
logger = logging.getLogger("aide")
logger.setLevel(logging.WARNING)


""" these dataclasses are just for type hinting, the actual config is in config.yaml """


@dataclass
class StageConfig:
    model: str
    temp: float


@dataclass
class SearchConfig:
    max_debug_depth: int
    debug_prob: float
    num_drafts: int


@dataclass
class AgentConfig:
    steps: int
    k_fold_validation: int
    expose_prediction: bool
    data_preview: bool

    code: StageConfig
    feedback: StageConfig

    search: SearchConfig


@dataclass
class ExecConfig:
    timeout: int
    agent_file_name: str
    format_tb_ipython: bool


@dataclass
class Config(Hashable):
    data_dir: Path
    desc_file: Path | None

    goal: str | None
    eval: str | None

    log_dir: Path
    workspace_dir: Path

    preprocess_data: bool
    copy_data: bool

    exp_name: str

    exec: ExecConfig
    agent: AgentConfig


def _get_next_logindex(dir: Path) -> int:
    """Get the next available index for a log directory."""
    max_index = -1
    for p in dir.iterdir():
        try:
            if current_index := int(p.name.split("-")[0]) > max_index:
                max_index = current_index
        except ValueError:
            pass
    return max_index + 1

def _load_cfg(path: Path = Path(__file__).parent / "config.yaml", use_cli_args=True) -> Config:
    cfg = OmegaConf.load(path)
    if use_cli_args:
        cfg = OmegaConf.merge(cfg, OmegaConf.from_cli())
    return cfg


def load_cfg(path: Path = Path(__file__).parent / "config.yaml") -> Config:
    """Load config from .yaml file and CLI args, and set up logging directory."""
    return prep_cfg(_load_cfg(path))


def prep_cfg(cfg: Config):
    if cfg.data_dir is None:
        raise ValueError("`data_dir` must be provided.")
    
    if cfg.desc_file is None and cfg.goal is None:
        raise ValueError("You must provide either a description of the task goal (`goal=...`) or a path to a plaintext file containing the description (`desc_file=...`).")

    if cfg.data_dir.startswith("example_tasks/"):
        cfg.data_dir = Path(__file__).parent.parent / cfg.data_dir
    cfg.data_dir = Path(cfg.data_dir).resolve()

    if cfg.desc_file is not None:
        cfg.desc_file = Path(cfg.desc_file).resolve()

    top_log_dir = Path(cfg.log_dir).resolve()
    top_log_dir.mkdir(parents=True, exist_ok=True)

    top_workspace_dir = Path(cfg.workspace_dir).resolve()
    top_workspace_dir.mkdir(parents=True, exist_ok=True)

    # generate experiment name and prefix with consecutive index
    ind = max(_get_next_logindex(top_log_dir), _get_next_logindex(top_workspace_dir))
    cfg.exp_name = cfg.exp_name or coolname.generate_slug(3)
    cfg.exp_name = f"{ind}-{cfg.exp_name}"

    cfg.log_dir = (top_log_dir / cfg.exp_name).resolve()
    cfg.workspace_dir = (top_workspace_dir / cfg.exp_name).resolve()

    # validate the config
    cfg_schema: Config = OmegaConf.structured(Config)
    cfg = OmegaConf.merge(cfg_schema, cfg)

    return cast(Config, cfg)


def print_cfg(cfg: Config) -> None:
    rich.print(Syntax(OmegaConf.to_yaml(cfg), "yaml", theme="paraiso-dark"))


def load_task_desc(cfg: Config):
    """Load task description from markdown file or config str."""

    # either load the task description from a file
    if cfg.desc_file is not None:
        if not (cfg.goal is None and cfg.eval is None):
            logger.warning(
                "Ignoring goal and eval args because task description file is provided."
            )
    
        with open() as f:
            return f.read()
        
    # or generate it from the goal and eval args
    if cfg.goal is None:
        raise ValueError("`goal` (and optionally `eval`) must be provided if a task description file is not provided.")

    task_desc = {"Task goal": cfg.goal}
    if cfg.eval is not None:
        task_desc["Task evaluation"] = cfg.eval

    return task_desc

def prep_agent_workspace(cfg: Config):
    """Setup the agent's workspace and preprocess data if necessary."""
    (cfg.workspace_dir / "input").mkdir(parents=True, exist_ok=True)
    (cfg.workspace_dir / "working").mkdir(parents=True, exist_ok=True)

    copytree(cfg.data_dir, cfg.workspace_dir / "input", use_symlinks=not cfg.copy_data)
    if cfg.preprocess_data:
        preproc_data(cfg.workspace_dir / "input")


def save_run(cfg: Config, journal):
    cfg.log_dir.mkdir(parents=True, exist_ok=True)
    
    # save journal
    serialize.dump_json(journal, cfg.log_dir / "journal.json")
    # save config
    OmegaConf.save(config=cfg, f=cfg.log_dir / "config.yaml")
    # create the tree + code visualization
    tree_export.generate(cfg, journal, cfg.log_dir / "tree_plot.html")
    # save the best found solution
    best_node = journal.get_best_node(only_good=False)
    with open(cfg.log_dir / "best_solution.py", "w") as f:
        f.write(best_node.code)



import pandas as pd
import lightgbm as lgb
from sklearn.model_selection import train_test_split, KFold
from sklearn.metrics import roc_auc_score
from sklearn.preprocessing import StandardScaler
from bayes_opt import BayesianOptimization

# Load the data
train_data = pd.read_csv("./input/train.csv")
test_data = pd.read_csv("./input/test.csv")

# Prepare the data
X = train_data.drop(["id", "smoking"], axis=1)
y = train_data["smoking"]
X_test = test_data.drop("id", axis=1)

# Split the data into training and validation sets
X_train, X_val, y_train, y_val = train_test_split(X, y, test_size=0.2, random_state=42)

# Scale the features
scaler = StandardScaler()
X_train_scaled = scaler.fit_transform(X_train)
X_val_scaled = scaler.transform(X_val)
X_test_scaled = scaler.transform(X_test)


# Define the LightGBM cross-validation function
def lgb_cv(
    learning_rate,
    num_leaves,
    min_child_samples,
    subsample,
    colsample_bytree,
    max_depth,
    reg_alpha,
    reg_lambda,
    n_estimators,
):
    params = {
        "objective": "binary",
        "metric": "auc",
        "boosting_type": "gbdt",
        "learning_rate": max(min(learning_rate, 1), 0),
        "n_estimators": int(n_estimators),
        "verbose": -1,
        "num_leaves": int(num_leaves),
        "min_child_samples": int(min_child_samples),
        "subsample": max(min(subsample, 1), 0),
        "colsample_bytree": max(min(colsample_bytree, 1), 0),
        "max_depth": int(max_depth),
        "reg_alpha": max(reg_alpha, 0),
        "reg_lambda": max(reg_lambda, 0),
    }
    cv_result = lgb.cv(
        params,
        lgb.Dataset(X_train_scaled, label=y_train),
        nfold=10,
        seed=42,
        stratified=True,
        verbose_eval=200,
        metrics=["auc"],
    )
    return max(cv_result["auc-mean"])


# Define the parameter bounds
param_bounds = {
    "learning_rate": (0.01, 0.2),
    "num_leaves": (20, 60),
    "min_child_samples": (5, 50),
    "subsample": (0.6, 1.0),
    "colsample_bytree": (0.6, 1.0),
    "max_depth": (5, 15),
    "reg_alpha": (0, 1),
    "reg_lambda": (0, 1),
    "n_estimators": (100, 1000),  # Increased range for n_estimators
}

# Perform Bayesian optimization with increased initial points and iterations
optimizer = BayesianOptimization(f=lgb_cv, pbounds=param_bounds, random_state=42)
optimizer.maximize(init_points=10, n_iter=50)

# Retrieve the best parameters
best_params = optimizer.max["params"]
best_params["num_leaves"] = int(best_params["num_leaves"])
best_params["min_child_samples"] = int(best_params["min_child_samples"])
best_params["max_depth"] = int(best_params["max_depth"])
best_params["n_estimators"] = int(best_params["n_estimators"])

# Train and validate the model with the best parameters
final_gbm = lgb.LGBMClassifier(**best_params)
final_gbm.fit(X_train_scaled, y_train)
val_predictions = final_gbm.predict_proba(X_val_scaled)[:, 1]
val_auc = roc_auc_score(y_val, val_predictions)
print(f"Validation AUC score: {val_auc}")

# Train the model on the full dataset with the best parameters and make predictions on the scaled test set
final_gbm.fit(scaler.fit_transform(X), y)
predictions = final_gbm.predict_proba(X_test_scaled)[:, 1]

# Prepare the submission file
submission = pd.DataFrame({"id": test_data["id"], "smoking": predictions})
submission.to_csv("./working/submission.csv", index=False)



"""
Python interpreter for executing code snippets and capturing their output.
Supports:
- captures stdout and stderr
- captures exceptions and stack traces
- limits execution time
"""

import logging
import os
import queue
import signal
import sys
import time
import traceback
from dataclasses import dataclass
from multiprocessing import Process, Queue
from pathlib import Path

import humanize
from dataclasses_json import DataClassJsonMixin

logger = logging.getLogger("aide")


@dataclass
class ExecutionResult(DataClassJsonMixin):
    """
    Result of executing a code snippet in the interpreter.
    Contains the output, execution time, and exception information.
    """

    term_out: list[str]
    exec_time: float
    exc_type: str | None
    exc_info: dict | None = None
    exc_stack: list[tuple] | None = None


def exception_summary(e, working_dir, exec_file_name, format_tb_ipython):
    """Generates a string that summarizes an exception and its stack trace (either in standard python repl or in IPython format)."""
    if format_tb_ipython:
        import IPython.core.ultratb

        # tb_offset = 1 to skip parts of the stack trace in weflow code
        tb = IPython.core.ultratb.VerboseTB(tb_offset=1, color_scheme="NoColor")
        tb_str = str(tb.text(*sys.exc_info()))
    else:
        tb_lines = traceback.format_exception(e)
        # skip parts of stack trace in weflow code
        tb_str = "".join(
            [l for l in tb_lines if "aide/" not in l and "importlib" not in l]
        )
        # tb_str = "".join([l for l in tb_lines])

    # replace whole path to file with just filename (to remove agent workspace dir)
    tb_str = tb_str.replace(str(working_dir / exec_file_name), exec_file_name)

    exc_info = {}
    if hasattr(e, "args"):
        exc_info["args"] = [str(i) for i in e.args]
    for att in ["name", "msg", "obj"]:
        if hasattr(e, att):
            exc_info[att] = str(getattr(e, att))

    tb = traceback.extract_tb(e.__traceback__)
    exc_stack = [(t.filename, t.lineno, t.name, t.line) for t in tb]

    return tb_str, e.__class__.__name__, exc_info, exc_stack


class RedirectQueue:
    def __init__(self, queue):
        self.queue = queue

    def write(self, msg):
        self.queue.put(msg)

    def flush(self):
        pass


class Interpreter:
    def __init__(
        self,
        working_dir: Path | str,
        timeout: int = 3600,
        format_tb_ipython: bool = False,
        agent_file_name: str = "runfile.py",
    ):
        """
        Simulates a standalone Python REPL with an execution time limit.

        Args:
            working_dir (Path | str): working directory of the agent
            timeout (int, optional): Timeout for each code execution step. Defaults to 3600.
            format_tb_ipython (bool, optional): Whether to use IPython or default python REPL formatting for exceptions. Defaults to False.
            agent_file_name (str, optional): The name for the agent's code file. Defaults to "runfile.py".
        """
        # this really needs to be a path, otherwise causes issues that don't raise exc
        self.working_dir = Path(working_dir).resolve()
        assert (
            self.working_dir.exists()
        ), f"Working directory {self.working_dir} does not exist"
        self.timeout = timeout
        self.format_tb_ipython = format_tb_ipython
        self.agent_file_name = agent_file_name
        self.process: Process = None  # type: ignore

    def child_proc_setup(self, result_outq: Queue) -> None:
        # disable all warnings (before importing anything)
        import shutup

        shutup.mute_warnings()
        os.chdir(str(self.working_dir))

        # this seems to only  benecessary because we're exec'ing code from a string,
        # a .py file should be able to import modules from the cwd anyway
        sys.path.append(str(self.working_dir))

        # capture stdout and stderr
        # trunk-ignore(mypy/assignment)
        sys.stdout = sys.stderr = RedirectQueue(result_outq)

    def _run_session(
        self, code_inq: Queue, result_outq: Queue, event_outq: Queue
    ) -> None:
        self.child_proc_setup(result_outq)

        global_scope: dict = {}
        while True:
            code = code_inq.get()
            os.chdir(str(self.working_dir))
            with open(self.agent_file_name, "w") as f:
                f.write(code)

            event_outq.put(("state:ready",))
            try:
                exec(compile(code, self.agent_file_name, "exec"), global_scope)
            except BaseException as e:
                tb_str, e_cls_name, exc_info, exc_stack = exception_summary(
                    e,
                    self.working_dir,
                    self.agent_file_name,
                    self.format_tb_ipython,
                )
                result_outq.put(tb_str)
                if e_cls_name == "KeyboardInterrupt":
                    e_cls_name = "TimeoutError"

                event_outq.put(("state:finished", e_cls_name, exc_info, exc_stack))
            else:
                event_outq.put(("state:finished", None, None, None))

            # remove the file after execution (otherwise it might be included in the data preview)
            os.remove(self.agent_file_name)

            # put EOF marker to indicate that we're done
            result_outq.put("<|EOF|>")

    def create_process(self) -> None:
        # we use three queues to communicate with the child process:
        # - code_inq: send code to child to execute
        # - result_outq: receive stdout/stderr from child
        # - event_outq: receive events from child (e.g. state:ready, state:finished)
        # trunk-ignore(mypy/var-annotated)
        self.code_inq, self.result_outq, self.event_outq = Queue(), Queue(), Queue()
        self.process = Process(
            target=self._run_session,
            args=(self.code_inq, self.result_outq, self.event_outq),
        )
        self.process.start()

    def cleanup_session(self):
        if self.process is None:
            return
        # give the child process a chance to terminate gracefully
        self.process.terminate()
        self.process.join(timeout=2)
        # kill the child process if it's still alive
        if self.process.exitcode is None:
            logger.warning("Child process failed to terminate gracefully, killing it..")
            self.process.kill()
            self.process.join()
        # don't wait for gc, clean up immediately
        self.process.close()
        self.process = None  # type: ignore

    def run(self, code: str, reset_session=True) -> ExecutionResult:
        """
        Execute the provided Python command in a separate process and return its output.

        Parameters:
            code (str): Python code to execute.
            reset_session (bool, optional): Whether to reset the interpreter session before executing the code. Defaults to True.

        Returns:
            ExecutionResult: Object containing the output and metadata of the code execution.

        """

        logger.debug(f"REPL is executing code (reset_session={reset_session})")

        if reset_session:
            if self.process is not None:
                # terminate and clean up previous process
                self.cleanup_session()
            self.create_process()
        else:
            # reset_session needs to be True on first exec
            assert self.process is not None

        assert self.process.is_alive()

        self.code_inq.put(code)

        # wait for child to actually start execution (we don't want interrupt child setup)
        try:
            state = self.event_outq.get(timeout=10)
        except queue.Empty:
            msg = "REPL child process failed to start execution"
            logger.critical(msg)
            while not self.result_outq.empty():
                logger.error(f"REPL output queue dump: {self.result_outq.get()}")
            raise RuntimeError(msg) from None
        assert state[0] == "state:ready", state
        start_time = time.time()

        # this flag indicates that the child ahs exceeded the time limit and an interrupt was sent
        # if the child process dies without this flag being set, it's an unexpected termination
        child_in_overtime = False

        while True:
            try:
                # check if the child is done
                state = self.event_outq.get(timeout=1)  # wait for state:finished
                assert state[0] == "state:finished", state
                exec_time = time.time() - start_time
                break
            except queue.Empty:
                # we haven't heard back from the child -> check if it's still alive (assuming overtime interrupt wasn't sent yet)
                if not child_in_overtime and not self.process.is_alive():
                    msg = "REPL child process died unexpectedly"
                    logger.critical(msg)
                    while not self.result_outq.empty():
                        logger.error(
                            f"REPL output queue dump: {self.result_outq.get()}"
                        )
                    raise RuntimeError(msg) from None

                # child is alive and still executing -> check if we should sigint..
                if self.timeout is None:
                    continue
                running_time = time.time() - start_time
                if running_time > self.timeout:

                    # [TODO] handle this in a better way
                    assert reset_session, "Timeout ocurred in interactive session"

                    # send interrupt to child
                    os.kill(self.process.pid, signal.SIGINT)  # type: ignore
                    child_in_overtime = True
                    # terminate if we're overtime by more than a minute
                    if running_time > self.timeout + 60:
                        logger.warning("Child failed to terminate, killing it..")
                        self.cleanup_session()

                        state = (None, "TimeoutError", {}, [])
                        exec_time = self.timeout
                        break

        output: list[str] = []
        # read all stdout/stderr from child up to the EOF marker
        # waiting until the queue is empty is not enough since
        # the feeder thread in child might still be adding to the queue
        while not self.result_outq.empty() or not output or output[-1] != "<|EOF|>":
            output.append(self.result_outq.get())
        output.pop()  # remove the EOF marker

        e_cls_name, exc_info, exc_stack = state[1:]

        if e_cls_name == "TimeoutError":
            output.append(
                f"TimeoutError: Execution exceeded the time limit of {humanize.naturaldelta(self.timeout)}"
            )
        else:
            output.append(
                f"Execution time: {humanize.naturaldelta(exec_time)} seconds (time limit is {humanize.naturaldelta(self.timeout)})."
            )
        return ExecutionResult(output, exec_time, e_cls_name, exc_info, exc_stack)



import pandas as pd
from sklearn.linear_model import LogisticRegression
from sklearn.model_selection import StratifiedKFold
from sklearn.metrics import roc_auc_score

# Load the data
train_data = pd.read_csv("./input/train.csv")
X_train = train_data.drop(["id", "target"], axis=1)
y_train = train_data["target"]

# Initialize the model with L1 regularization
model = LogisticRegression(penalty="l1", solver="liblinear", random_state=42)

# Prepare cross-validation
cv = StratifiedKFold(n_splits=10, shuffle=True, random_state=42)
auc_scores = []

# Perform 10-fold cross-validation
for train_idx, valid_idx in cv.split(X_train, y_train):
    X_train_fold, X_valid_fold = X_train.iloc[train_idx], X_train.iloc[valid_idx]
    y_train_fold, y_valid_fold = y_train.iloc[train_idx], y_train.iloc[valid_idx]

    # Train the model
    model.fit(X_train_fold, y_train_fold)

    # Predict probabilities for the validation set
    y_pred_prob = model.predict_proba(X_valid_fold)[:, 1]

    # Calculate the AUC score and append to the list
    auc_score = roc_auc_score(y_valid_fold, y_pred_prob)
    auc_scores.append(auc_score)

# Calculate the average AUC score across all folds
average_auc_score = sum(auc_scores) / len(auc_scores)
print(f"Average AUC-ROC score: {average_auc_score}")

# Train the model on the full training set and predict for the test set
model.fit(X_train, y_train)
test_data = pd.read_csv("./input/test.csv")
X_test = test_data.drop("id", axis=1)
test_data["target"] = model.predict_proba(X_test)[:, 1]

# Save the submission file
submission_file = "./working/submission.csv"
test_data[["id", "target"]].to_csv(submission_file, index=False)
